# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: whylogs_messages.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16whylogs_messages.proto\x1a\x19google/protobuf/any.proto\x1a\x1cgoogle/protobuf/struct.proto\"~\n\x08\x44\x61taType\x12\x1c\n\x04type\x18\x01 \x01(\x0e\x32\x0e.DataType.Type\"T\n\x04Type\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04NULL\x10\x01\x12\x0e\n\nFRACTIONAL\x10\x02\x12\x0c\n\x08INTEGRAL\x10\x03\x12\x0b\n\x07\x42OOLEAN\x10\x04\x12\n\n\x06STRING\x10\x05\"\"\n\x10HllSketchMessage\x12\x0e\n\x06sketch\x18\x01 \x01(\x0c\",\n\x1a\x46requentItemsSketchMessage\x12\x0e\n\x06sketch\x18\x01 \x01(\x0c\"\"\n\x10KllSketchMessage\x12\x0e\n\x06sketch\x18\x01 \x01(\x0c\"\"\n\x10\x43pcSketchMessage\x12\x0e\n\x06sketch\x18\x01 \x01(\x0c\"\xde\x02\n\x16MetricComponentMessage\x12\x0f\n\x07type_id\x18\x01 \x01(\r\x12\x0b\n\x01n\x18\x02 \x01(\x03H\x00\x12\x0b\n\x01\x64\x18\x03 \x01(\x01H\x00\x12\x35\n\x0e\x66requent_items\x18\x04 \x01(\x0b\x32\x1b.FrequentItemsSketchMessageH\x00\x12 \n\x03hll\x18\x05 \x01(\x0b\x32\x11.HllSketchMessageH\x00\x12 \n\x03kll\x18\x06 \x01(\x0b\x32\x11.KllSketchMessageH\x00\x12 \n\x03\x63pc\x18\x07 \x01(\x0b\x32\x11.CpcSketchMessageH\x00\x12\x1a\n\x10serialized_bytes\x18\n \x01(\x0cH\x00\x12\x32\n\x0f\x64\x61taclass_param\x18\x0b \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x12#\n\x03msg\x18\x0c \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x42\x07\n\x05value\"\xa2\x01\n\rMetricMessage\x12?\n\x11metric_components\x18\x01 \x03(\x0b\x32$.MetricMessage.MetricComponentsEntry\x1aP\n\x15MetricComponentsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.MetricComponentMessage:\x02\x38\x01\"\xa2\x01\n\rColumnMessage\x12?\n\x11metric_components\x18\x01 \x03(\x0b\x32$.ColumnMessage.MetricComponentsEntry\x1aP\n\x15MetricComponentsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.MetricComponentMessage:\x02\x38\x01\"\xc4\x02\n\x11\x44\x61tasetProperties\x12\x1c\n\x14schema_major_version\x18\x01 \x01(\r\x12\x1c\n\x14schema_minor_version\x18\x02 \x01(\r\x12\x1a\n\x12\x63reation_timestamp\x18\x04 \x01(\x04\x12\x19\n\x11\x64\x61taset_timestamp\x18\x05 \x01(\x04\x12*\n\x04tags\x18\x06 \x03(\x0b\x32\x1c.DatasetProperties.TagsEntry\x12\x32\n\x08metadata\x18\x07 \x03(\x0b\x32 .DatasetProperties.MetadataEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x1f\n\x0c\x43hunkOffsets\x12\x0f\n\x07offsets\x18\x01 \x03(\x04\"\xa0\x01\n\x0c\x43hunkMessage\x12>\n\x11metric_components\x18\x01 \x03(\x0b\x32#.ChunkMessage.MetricComponentsEntry\x1aP\n\x15MetricComponentsEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.MetricComponentMessage:\x02\x38\x01\"v\n\x0b\x43hunkHeader\x12$\n\x04type\x18\x01 \x01(\x0e\x32\x16.ChunkHeader.ChunkType\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x0e\n\x06length\x18\x03 \x01(\r\"$\n\tChunkType\x12\x0b\n\x07\x44\x41TASET\x10\x00\x12\n\n\x06\x43OLUMN\x10\x02\"\x84\x03\n\x14\x44\x61tasetProfileHeader\x12&\n\nproperties\x18\x01 \x01(\x0b\x32\x12.DatasetProperties\x12@\n\x0e\x63olumn_offsets\x18\x02 \x03(\x0b\x32(.DatasetProfileHeader.ColumnOffsetsEntry\x12%\n\x0emetric_offsets\x18\x03 \x03(\x0b\x32\r.ChunkOffsets\x12\x0e\n\x06length\x18\x04 \x01(\x04\x12K\n\x14indexed_metric_paths\x18\x05 \x03(\x0b\x32-.DatasetProfileHeader.IndexedMetricPathsEntry\x1a\x43\n\x12\x43olumnOffsetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1c\n\x05value\x18\x02 \x01(\x0b\x32\r.ChunkOffsets:\x02\x38\x01\x1a\x39\n\x17IndexedMetricPathsEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"(\n\nSegmentTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\":\n\x07Segment\x12\x19\n\x04tags\x18\x01 \x03(\x0b\x32\x0b.SegmentTag\x12\x14\n\x0cpartition_id\x18\x02 \x01(\t\"1\n\x15SegmentationPartition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\"\xad\x01\n\x14\x44\x61tasetSegmentHeader\x12\x14\n\x0chas_segments\x18\x01 \x01(\x08\x12\x1a\n\x08segments\x18\x04 \x03(\x0b\x32\x08.Segment\x12\x33\n\x07offsets\x18\x05 \x03(\x0b\x32\".DatasetSegmentHeader.OffsetsEntry\x1a.\n\x0cOffsetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\x04:\x02\x38\x01\x42&\n\x18\x63om.whylogs.core.messageB\x08MessagesP\x01\x62\x06proto3')



_DATATYPE = DESCRIPTOR.message_types_by_name['DataType']
_HLLSKETCHMESSAGE = DESCRIPTOR.message_types_by_name['HllSketchMessage']
_FREQUENTITEMSSKETCHMESSAGE = DESCRIPTOR.message_types_by_name['FrequentItemsSketchMessage']
_KLLSKETCHMESSAGE = DESCRIPTOR.message_types_by_name['KllSketchMessage']
_CPCSKETCHMESSAGE = DESCRIPTOR.message_types_by_name['CpcSketchMessage']
_METRICCOMPONENTMESSAGE = DESCRIPTOR.message_types_by_name['MetricComponentMessage']
_METRICMESSAGE = DESCRIPTOR.message_types_by_name['MetricMessage']
_METRICMESSAGE_METRICCOMPONENTSENTRY = _METRICMESSAGE.nested_types_by_name['MetricComponentsEntry']
_COLUMNMESSAGE = DESCRIPTOR.message_types_by_name['ColumnMessage']
_COLUMNMESSAGE_METRICCOMPONENTSENTRY = _COLUMNMESSAGE.nested_types_by_name['MetricComponentsEntry']
_DATASETPROPERTIES = DESCRIPTOR.message_types_by_name['DatasetProperties']
_DATASETPROPERTIES_TAGSENTRY = _DATASETPROPERTIES.nested_types_by_name['TagsEntry']
_DATASETPROPERTIES_METADATAENTRY = _DATASETPROPERTIES.nested_types_by_name['MetadataEntry']
_CHUNKOFFSETS = DESCRIPTOR.message_types_by_name['ChunkOffsets']
_CHUNKMESSAGE = DESCRIPTOR.message_types_by_name['ChunkMessage']
_CHUNKMESSAGE_METRICCOMPONENTSENTRY = _CHUNKMESSAGE.nested_types_by_name['MetricComponentsEntry']
_CHUNKHEADER = DESCRIPTOR.message_types_by_name['ChunkHeader']
_DATASETPROFILEHEADER = DESCRIPTOR.message_types_by_name['DatasetProfileHeader']
_DATASETPROFILEHEADER_COLUMNOFFSETSENTRY = _DATASETPROFILEHEADER.nested_types_by_name['ColumnOffsetsEntry']
_DATASETPROFILEHEADER_INDEXEDMETRICPATHSENTRY = _DATASETPROFILEHEADER.nested_types_by_name['IndexedMetricPathsEntry']
_SEGMENTTAG = DESCRIPTOR.message_types_by_name['SegmentTag']
_SEGMENT = DESCRIPTOR.message_types_by_name['Segment']
_SEGMENTATIONPARTITION = DESCRIPTOR.message_types_by_name['SegmentationPartition']
_DATASETSEGMENTHEADER = DESCRIPTOR.message_types_by_name['DatasetSegmentHeader']
_DATASETSEGMENTHEADER_OFFSETSENTRY = _DATASETSEGMENTHEADER.nested_types_by_name['OffsetsEntry']
_DATATYPE_TYPE = _DATATYPE.enum_types_by_name['Type']
_CHUNKHEADER_CHUNKTYPE = _CHUNKHEADER.enum_types_by_name['ChunkType']
DataType = _reflection.GeneratedProtocolMessageType('DataType', (_message.Message,), {
  'DESCRIPTOR' : _DATATYPE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:DataType)
  })
_sym_db.RegisterMessage(DataType)

HllSketchMessage = _reflection.GeneratedProtocolMessageType('HllSketchMessage', (_message.Message,), {
  'DESCRIPTOR' : _HLLSKETCHMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:HllSketchMessage)
  })
_sym_db.RegisterMessage(HllSketchMessage)

FrequentItemsSketchMessage = _reflection.GeneratedProtocolMessageType('FrequentItemsSketchMessage', (_message.Message,), {
  'DESCRIPTOR' : _FREQUENTITEMSSKETCHMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:FrequentItemsSketchMessage)
  })
_sym_db.RegisterMessage(FrequentItemsSketchMessage)

KllSketchMessage = _reflection.GeneratedProtocolMessageType('KllSketchMessage', (_message.Message,), {
  'DESCRIPTOR' : _KLLSKETCHMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:KllSketchMessage)
  })
_sym_db.RegisterMessage(KllSketchMessage)

CpcSketchMessage = _reflection.GeneratedProtocolMessageType('CpcSketchMessage', (_message.Message,), {
  'DESCRIPTOR' : _CPCSKETCHMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:CpcSketchMessage)
  })
_sym_db.RegisterMessage(CpcSketchMessage)

MetricComponentMessage = _reflection.GeneratedProtocolMessageType('MetricComponentMessage', (_message.Message,), {
  'DESCRIPTOR' : _METRICCOMPONENTMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:MetricComponentMessage)
  })
_sym_db.RegisterMessage(MetricComponentMessage)

MetricMessage = _reflection.GeneratedProtocolMessageType('MetricMessage', (_message.Message,), {

  'MetricComponentsEntry' : _reflection.GeneratedProtocolMessageType('MetricComponentsEntry', (_message.Message,), {
    'DESCRIPTOR' : _METRICMESSAGE_METRICCOMPONENTSENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:MetricMessage.MetricComponentsEntry)
    })
  ,
  'DESCRIPTOR' : _METRICMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:MetricMessage)
  })
_sym_db.RegisterMessage(MetricMessage)
_sym_db.RegisterMessage(MetricMessage.MetricComponentsEntry)

ColumnMessage = _reflection.GeneratedProtocolMessageType('ColumnMessage', (_message.Message,), {

  'MetricComponentsEntry' : _reflection.GeneratedProtocolMessageType('MetricComponentsEntry', (_message.Message,), {
    'DESCRIPTOR' : _COLUMNMESSAGE_METRICCOMPONENTSENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:ColumnMessage.MetricComponentsEntry)
    })
  ,
  'DESCRIPTOR' : _COLUMNMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:ColumnMessage)
  })
_sym_db.RegisterMessage(ColumnMessage)
_sym_db.RegisterMessage(ColumnMessage.MetricComponentsEntry)

DatasetProperties = _reflection.GeneratedProtocolMessageType('DatasetProperties', (_message.Message,), {

  'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETPROPERTIES_TAGSENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetProperties.TagsEntry)
    })
  ,

  'MetadataEntry' : _reflection.GeneratedProtocolMessageType('MetadataEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETPROPERTIES_METADATAENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetProperties.MetadataEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETPROPERTIES,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:DatasetProperties)
  })
_sym_db.RegisterMessage(DatasetProperties)
_sym_db.RegisterMessage(DatasetProperties.TagsEntry)
_sym_db.RegisterMessage(DatasetProperties.MetadataEntry)

ChunkOffsets = _reflection.GeneratedProtocolMessageType('ChunkOffsets', (_message.Message,), {
  'DESCRIPTOR' : _CHUNKOFFSETS,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:ChunkOffsets)
  })
_sym_db.RegisterMessage(ChunkOffsets)

ChunkMessage = _reflection.GeneratedProtocolMessageType('ChunkMessage', (_message.Message,), {

  'MetricComponentsEntry' : _reflection.GeneratedProtocolMessageType('MetricComponentsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CHUNKMESSAGE_METRICCOMPONENTSENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:ChunkMessage.MetricComponentsEntry)
    })
  ,
  'DESCRIPTOR' : _CHUNKMESSAGE,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:ChunkMessage)
  })
_sym_db.RegisterMessage(ChunkMessage)
_sym_db.RegisterMessage(ChunkMessage.MetricComponentsEntry)

ChunkHeader = _reflection.GeneratedProtocolMessageType('ChunkHeader', (_message.Message,), {
  'DESCRIPTOR' : _CHUNKHEADER,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:ChunkHeader)
  })
_sym_db.RegisterMessage(ChunkHeader)

DatasetProfileHeader = _reflection.GeneratedProtocolMessageType('DatasetProfileHeader', (_message.Message,), {

  'ColumnOffsetsEntry' : _reflection.GeneratedProtocolMessageType('ColumnOffsetsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETPROFILEHEADER_COLUMNOFFSETSENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetProfileHeader.ColumnOffsetsEntry)
    })
  ,

  'IndexedMetricPathsEntry' : _reflection.GeneratedProtocolMessageType('IndexedMetricPathsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETPROFILEHEADER_INDEXEDMETRICPATHSENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetProfileHeader.IndexedMetricPathsEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETPROFILEHEADER,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:DatasetProfileHeader)
  })
_sym_db.RegisterMessage(DatasetProfileHeader)
_sym_db.RegisterMessage(DatasetProfileHeader.ColumnOffsetsEntry)
_sym_db.RegisterMessage(DatasetProfileHeader.IndexedMetricPathsEntry)

SegmentTag = _reflection.GeneratedProtocolMessageType('SegmentTag', (_message.Message,), {
  'DESCRIPTOR' : _SEGMENTTAG,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:SegmentTag)
  })
_sym_db.RegisterMessage(SegmentTag)

Segment = _reflection.GeneratedProtocolMessageType('Segment', (_message.Message,), {
  'DESCRIPTOR' : _SEGMENT,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:Segment)
  })
_sym_db.RegisterMessage(Segment)

SegmentationPartition = _reflection.GeneratedProtocolMessageType('SegmentationPartition', (_message.Message,), {
  'DESCRIPTOR' : _SEGMENTATIONPARTITION,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:SegmentationPartition)
  })
_sym_db.RegisterMessage(SegmentationPartition)

DatasetSegmentHeader = _reflection.GeneratedProtocolMessageType('DatasetSegmentHeader', (_message.Message,), {

  'OffsetsEntry' : _reflection.GeneratedProtocolMessageType('OffsetsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETSEGMENTHEADER_OFFSETSENTRY,
    '__module__' : 'whylogs_messages_pb2'
    # @@protoc_insertion_point(class_scope:DatasetSegmentHeader.OffsetsEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETSEGMENTHEADER,
  '__module__' : 'whylogs_messages_pb2'
  # @@protoc_insertion_point(class_scope:DatasetSegmentHeader)
  })
_sym_db.RegisterMessage(DatasetSegmentHeader)
_sym_db.RegisterMessage(DatasetSegmentHeader.OffsetsEntry)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030com.whylogs.core.messageB\010MessagesP\001'
  _METRICMESSAGE_METRICCOMPONENTSENTRY._options = None
  _METRICMESSAGE_METRICCOMPONENTSENTRY._serialized_options = b'8\001'
  _COLUMNMESSAGE_METRICCOMPONENTSENTRY._options = None
  _COLUMNMESSAGE_METRICCOMPONENTSENTRY._serialized_options = b'8\001'
  _DATASETPROPERTIES_TAGSENTRY._options = None
  _DATASETPROPERTIES_TAGSENTRY._serialized_options = b'8\001'
  _DATASETPROPERTIES_METADATAENTRY._options = None
  _DATASETPROPERTIES_METADATAENTRY._serialized_options = b'8\001'
  _CHUNKMESSAGE_METRICCOMPONENTSENTRY._options = None
  _CHUNKMESSAGE_METRICCOMPONENTSENTRY._serialized_options = b'8\001'
  _DATASETPROFILEHEADER_COLUMNOFFSETSENTRY._options = None
  _DATASETPROFILEHEADER_COLUMNOFFSETSENTRY._serialized_options = b'8\001'
  _DATASETPROFILEHEADER_INDEXEDMETRICPATHSENTRY._options = None
  _DATASETPROFILEHEADER_INDEXEDMETRICPATHSENTRY._serialized_options = b'8\001'
  _DATASETSEGMENTHEADER_OFFSETSENTRY._options = None
  _DATASETSEGMENTHEADER_OFFSETSENTRY._serialized_options = b'8\001'
  _DATATYPE._serialized_start=83
  _DATATYPE._serialized_end=209
  _DATATYPE_TYPE._serialized_start=125
  _DATATYPE_TYPE._serialized_end=209
  _HLLSKETCHMESSAGE._serialized_start=211
  _HLLSKETCHMESSAGE._serialized_end=245
  _FREQUENTITEMSSKETCHMESSAGE._serialized_start=247
  _FREQUENTITEMSSKETCHMESSAGE._serialized_end=291
  _KLLSKETCHMESSAGE._serialized_start=293
  _KLLSKETCHMESSAGE._serialized_end=327
  _CPCSKETCHMESSAGE._serialized_start=329
  _CPCSKETCHMESSAGE._serialized_end=363
  _METRICCOMPONENTMESSAGE._serialized_start=366
  _METRICCOMPONENTMESSAGE._serialized_end=716
  _METRICMESSAGE._serialized_start=719
  _METRICMESSAGE._serialized_end=881
  _METRICMESSAGE_METRICCOMPONENTSENTRY._serialized_start=801
  _METRICMESSAGE_METRICCOMPONENTSENTRY._serialized_end=881
  _COLUMNMESSAGE._serialized_start=884
  _COLUMNMESSAGE._serialized_end=1046
  _COLUMNMESSAGE_METRICCOMPONENTSENTRY._serialized_start=801
  _COLUMNMESSAGE_METRICCOMPONENTSENTRY._serialized_end=881
  _DATASETPROPERTIES._serialized_start=1049
  _DATASETPROPERTIES._serialized_end=1373
  _DATASETPROPERTIES_TAGSENTRY._serialized_start=1281
  _DATASETPROPERTIES_TAGSENTRY._serialized_end=1324
  _DATASETPROPERTIES_METADATAENTRY._serialized_start=1326
  _DATASETPROPERTIES_METADATAENTRY._serialized_end=1373
  _CHUNKOFFSETS._serialized_start=1375
  _CHUNKOFFSETS._serialized_end=1406
  _CHUNKMESSAGE._serialized_start=1409
  _CHUNKMESSAGE._serialized_end=1569
  _CHUNKMESSAGE_METRICCOMPONENTSENTRY._serialized_start=1489
  _CHUNKMESSAGE_METRICCOMPONENTSENTRY._serialized_end=1569
  _CHUNKHEADER._serialized_start=1571
  _CHUNKHEADER._serialized_end=1689
  _CHUNKHEADER_CHUNKTYPE._serialized_start=1653
  _CHUNKHEADER_CHUNKTYPE._serialized_end=1689
  _DATASETPROFILEHEADER._serialized_start=1692
  _DATASETPROFILEHEADER._serialized_end=2080
  _DATASETPROFILEHEADER_COLUMNOFFSETSENTRY._serialized_start=1954
  _DATASETPROFILEHEADER_COLUMNOFFSETSENTRY._serialized_end=2021
  _DATASETPROFILEHEADER_INDEXEDMETRICPATHSENTRY._serialized_start=2023
  _DATASETPROFILEHEADER_INDEXEDMETRICPATHSENTRY._serialized_end=2080
  _SEGMENTTAG._serialized_start=2082
  _SEGMENTTAG._serialized_end=2122
  _SEGMENT._serialized_start=2124
  _SEGMENT._serialized_end=2182
  _SEGMENTATIONPARTITION._serialized_start=2184
  _SEGMENTATIONPARTITION._serialized_end=2233
  _DATASETSEGMENTHEADER._serialized_start=2236
  _DATASETSEGMENTHEADER._serialized_end=2409
  _DATASETSEGMENTHEADER_OFFSETSENTRY._serialized_start=2363
  _DATASETSEGMENTHEADER_OFFSETSENTRY._serialized_end=2409
# @@protoc_insertion_point(module_scope)



import atexit
import hashlib
import http.client
import json
import logging
import os
import site
import socket
import sys
import uuid
from datetime import datetime
from threading import Thread
from typing import Any, Dict
from urllib import request

import whylogs

_TELEMETRY_ENDPOINT = "https://stats.whylogs.com/"
if os.getenv("TELEMETRY_DEV"):
    _TELEMETRY_ENDPOINT = "https://staging-stats.whylogs.com"
_TIMESTAMP_FORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"
logger = logging.getLogger(__name__)

ANALYTICS_OPT_OUT = "WHYLOGS_NO_ANALYTICS"

# Flag to disable it internally
_TELEMETRY_DISABLED = False
_TRACKED_EVENTS: Dict[str, bool] = {}

_SITE_PACKAGES = site.getsitepackages()

if os.getenv(ANALYTICS_OPT_OUT) is not None:
    logger.debug("Opted out of usage statistics. Skipping.")
    _TELEMETRY_DISABLED = True

try:
    if os.path.exists(os.path.expanduser("~/.whylogs/disable_telemetry")):
        _TELEMETRY_DISABLED = True
except:  # noqa
    logger.info("Encounter exception when checking file system. Disable telemetry by default")
    _TELEMETRY_DISABLED = True


def emit_usage(event: str) -> None:
    global _TELEMETRY_DISABLED
    global _TRACKED_EVENTS
    if _TELEMETRY_DISABLED:
        return
    if _TRACKED_EVENTS.get(event):
        return
    _TRACKED_EVENTS[event] = True

    t = Thread(target=_do_emit_usage, args=(event,))
    t.start()

    atexit.register(t.join)


_metadata = None
_identity = None


def _do_emit_usage(event: str) -> None:
    global _TELEMETRY_DISABLED
    if _TELEMETRY_DISABLED:
        logger.debug("Opted out of usage statistics. Skipping.")
        return

    logger.debug("Telemetry opted in. Emitting usage statistics")

    global _identity
    global _metadata
    if _identity is None:
        _identity = _calc_identity()
    if _metadata is None:
        _metadata = _build_metadata()

    _send_stats_event(event, _identity, _metadata)


def _calc_identity() -> str:
    try:
        hashed_computer_name = hashlib.sha512(bytes(socket.gethostname(), encoding="utf8"))
        return hashed_computer_name.hexdigest()
    except socket.timeout as exc:
        logger.debug(
            "Socket timeout when trying to get the computer name. Exception: %s",
            exc,
        )
        return uuid.uuid4().hex


def _build_metadata() -> Dict[str, Any]:
    """Hash system and project data to send to our stats endpoint."""

    project_version = whylogs.__version__
    (major, minor, macro, _, _) = sys.version_info

    metadata = {
        "project_version": project_version,
        "python_version": f"{major}.{minor}.{macro}",
        "python_version_full": sys.version,
        "terminal": _get_terminal_mode(),
        "os": sys.platform,
        "conda": ("CONDA_DEFAULT_ENV" in os.environ),
        "venv": ("VIRTUAL_ENV" in os.environ),
        "environment": _get_environment(),
    }

    # track various integrations
    integrations = {
        "numpy": _has_lib("numpy"),
        "pandas": _has_lib("pandas"),
        "mlflow": _has_lib("mlflow"),
        "dask": _has_lib("dask"),
        "ray": _has_lib("ray"),
        "airflow": _has_lib("airflow"),
        "pyspark": _has_lib("pyspark"),
        "flyte": _has_lib("flyte"),
        "kafka": _has_lib("kafka"),
    }
    for k in list(integrations.keys()):
        if integrations.get(k) is False:
            integrations.pop(k)

    # add integration metadata
    metadata.update(integrations)
    return metadata


def _send_stats_event(event_name: str, identity: str, properties: Dict[str, Any] = None) -> None:
    data = {
        "identity": identity,
        "event": event_name,
        "timestamp": datetime.utcnow().strftime(_TIMESTAMP_FORMAT),
        "properties": properties or {},
    }
    global _TELEMETRY_DISABLED
    json_data = json.dumps(data).encode()
    req = request.Request(_TELEMETRY_ENDPOINT, data=json_data, method="POST")
    req.add_header("Content-Type", "application/json")

    resp: http.client.HTTPResponse = None  # type: ignore
    try:
        resp = request.urlopen(req, timeout=3)
        if resp.status != 200:
            logger.warning("Unable to send usage stats. Disabling stats collection.")
            _TELEMETRY_DISABLED = True
        logger.debug("Response: %s", resp.read())
    except:  # noqa
        logger.warning("Connection error. Skip stats collection.")
        _TELEMETRY_DISABLED = True

    finally:
        if resp is not None:
            resp.close()


def _get_terminal_mode() -> str:
    try:
        from IPython.core.getipython import get_ipython  # type: ignore

        ipython = get_ipython()
        if ipython is not None:
            return ipython.__class__.__name__
    except:  # noqa
        pass

    if hasattr(sys, "ps1"):
        return "shell"
    else:
        return "headless"


def _get_environment() -> str:
    environments_dict = {
        "GITHUB_ACTION": "github_action",
        "GITLAB_CI": "gitlab_ci",
        "BINDER_PORT": "binder",
        "PYCHARM_HOSTED": "pycharm",
        "SM_CURRENT_HOST": "sagemaker",
        "DATABRICKS_RUNTIME_VERSION": "databricks",
        "COLAB_GPU": "colab",
        "KAGGLE_KERNEL_RUN_TYPE": "kaggle",
        "DEEPNOTE_PROJECT_ID": "deepnote",
    }

    for key, value in environments_dict.items():
        if key in os.environ:
            return value
    return "unknown"


def _has_lib(lib_name: str) -> bool:
    try:
        for p in _SITE_PACKAGES:
            if os.path.exists(os.path.join(p, lib_name)):
                return True
    except:  # noqa
        pass

    return False



import atexit
import logging
from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional

from whylogs.api.logger.result_set import ProfileResultSet, ResultSet
from whylogs.api.logger.segment_processing import segment_processing
from whylogs.api.writer import Writer, Writers
from whylogs.core import DatasetProfile, DatasetSchema
from whylogs.core.errors import LoggingError
from whylogs.core.stubs import pd

logger = logging.getLogger(__name__)


class BasicCache(object):
    _cache: Dict[DatasetSchema, DatasetProfile] = {}

    def get(self, schema: DatasetSchema) -> DatasetProfile:
        candidate = self._cache.get(schema)
        if candidate is None:
            candidate = DatasetProfile(schema=schema)
            self._cache[schema] = candidate
        return candidate


class Logger(ABC):
    def __init__(self, schema: Optional[DatasetSchema] = None):
        self._is_closed = False
        self._schema = schema
        self._writers: List[Writer] = []
        atexit.register(self.close)

    def check_writer(self, _: Writer) -> None:
        """Checks if a writer is configured correctly for this class"""

    def append_writer(self, name: Optional[str] = None, *, writer: Optional[Writer] = None, **kwargs: Any) -> None:
        if name is None and writer is None:
            raise ValueError("Must specify either the writer name or a Writer object")
        if name is not None and writer is not None:
            raise ValueError("Cannot specify name and writer at the same time")
        if name is not None:
            writer = Writers.get(name, **kwargs)

        assert writer is not None
        self.check_writer(writer)
        self._writers.append(writer)

    @abstractmethod
    def _get_matching_profiles(
        self,
        obj: Any = None,
        *,
        pandas: Optional[pd.DataFrame] = None,
        row: Optional[Dict[str, Any]] = None,
    ) -> List[DatasetProfile]:
        pass

    def log(
        self,
        obj: Any = None,
        *,
        pandas: Optional[pd.DataFrame] = None,
        row: Optional[Dict[str, Any]] = None,
    ) -> ResultSet:
        if self._is_closed:
            raise LoggingError("Cannot log to a closed logger")
        if obj is None and pandas is None and row is None:
            # TODO: check for shell environment and emit more verbose error string to let user know how to correct.
            raise LoggingError("log() was called without passing in any input!")

        # If segments are defined use segment_processing to return a SegmentedResultSet
        if self._schema and self._schema.segments:
            return segment_processing(self._schema, obj, pandas, row)

        profiles = self._get_matching_profiles(obj, pandas=pandas, row=row)

        for prof in profiles:
            prof.track(obj, pandas=pandas, row=row)

        return ProfileResultSet(profiles[0])

    @abstractmethod
    def close(self) -> None:
        self._is_closed = True

    def __enter__(self) -> "Logger":
        return self

    def __exit__(self, exception_type: Any, exception_value: Any, traceback: Any) -> None:
        if not self._is_closed:
            self.close()



from dataclasses import dataclass, field
from enum import Enum
from typing import List

import whylogs_sketching as ds  # type: ignore


class FrequentItemsErrorType(int, Enum):
    NO_FALSE_NEGATIVES = 1
    NO_FALSE_POSITIVES = 0

    def to_datasketches_type(self) -> ds.frequent_items_error_type:
        if self.value == FrequentItemsErrorType.NO_FALSE_NEGATIVES.value:
            return ds.frequent_items_error_type.NO_FALSE_NEGATIVES
        else:
            return ds.frequent_items_error_type.NO_FALSE_POSITIVES


@dataclass
class SummaryConfig:
    disabled_metrics: List[str] = field(default_factory=list)
    frequent_items_error_type: FrequentItemsErrorType = FrequentItemsErrorType.NO_FALSE_POSITIVES
    frequent_items_limit: int = -1
    hll_stddev: int = 1



from typing import Any, Callable, Dict, Optional, TypeVar

import whylogs_sketching as ds  # type: ignore

from whylogs.core.metrics.decorators import (
    DecoratedFunction,
    FuncType,
    _decorate_func,
    _func_wrapper,
)
from whylogs.core.proto import MetricComponentMessage

M = TypeVar("M")
NUM = TypeVar("NUM", float, int)

_MAX_BUILT_IN_ID = 100


class _Deserializer(DecoratedFunction):
    def __init__(self, *, func: FuncType, name: str):  # noqa
        self._name = name
        self._func = func

    @property
    def name(self) -> Optional[str]:
        return self._name

    def __call__(self, *, msg: MetricComponentMessage) -> Any:
        return self._func(msg)

    @classmethod
    def build(cls, func: FuncType, name: str) -> "_Deserializer":  # noqa
        return _Deserializer(func=func, name=name)


# these are all built in and initialized at imports
_TYPED_DESERIALIZERS: Dict[type, _Deserializer] = {}
_ID_DESERIALIZERS: Dict[int, _Deserializer] = {}


def _builtin_deserializer(*, name: str) -> Callable[[Callable], _Deserializer]:
    """Decorator for a builtin field aggregator.

    Note that since these are built in, they MUST be unique and stable over time.
    Args:
        name: a human readable string. This must be unique for validation.

    """

    def decorated(func: FuncType) -> _Deserializer:
        annotations: Dict[str, type] = func.__annotations__.copy()
        r_type = annotations.pop("return")

        arg_len = len(annotations.items())
        if arg_len != 1:
            raise ValueError(f"Expected 1 argument, got: {arg_len}")

        if next(iter(annotations.values())) != MetricComponentMessage:
            raise ValueError("Invalid function type: input argument not MetricComponentMessage")

        return _func_wrapper(
            func=func, key=r_type, name=f"builtin.{r_type}", wrapper_dict=_TYPED_DESERIALIZERS, clazz=_Deserializer
        )

    return decorated  # type: ignore


@_builtin_deserializer(name="int")
def _int(msg: MetricComponentMessage) -> int:
    field = msg.WhichOneof("value")
    if field != "n":
        raise ValueError(f"Unsupported field: {field}")
    return msg.n


@_builtin_deserializer(name="float")
def _float(msg: MetricComponentMessage) -> float:
    field = msg.WhichOneof("value")
    if field != "d":
        raise ValueError(f"Unsupported field: {field}")
    return msg.d


@_builtin_deserializer(name="kll")
def _kll(msg: MetricComponentMessage) -> ds.kll_doubles_sketch:
    field = msg.WhichOneof("value")
    if field != "kll":
        raise ValueError(f"Unsupported field: {field}")

    return ds.kll_doubles_sketch.deserialize(msg.kll.sketch)


@_builtin_deserializer(name="kll")
def _hll_merge(msg: MetricComponentMessage) -> ds.hll_sketch:
    field = msg.WhichOneof("value")
    if field != "hll":
        raise ValueError(f"Unsupported field: {field}")

    sketch = ds.hll_sketch.deserialize(msg.hll.sketch)
    return sketch


@_builtin_deserializer(name="kll")
def _fs_merge(msg: MetricComponentMessage) -> ds.frequent_strings_sketch:
    field = msg.WhichOneof("value")
    if field != "frequent_items":
        raise ValueError(f"Unsupported field: {field}")

    return ds.frequent_strings_sketch.deserialize(msg.frequent_items.sketch)


class DeserializerRegistry:
    def __init__(self) -> None:
        self._typed_deserializer = _TYPED_DESERIALIZERS.copy()
        self._id_deserializer = _ID_DESERIALIZERS.copy()

    def get(self, *, mtype: Optional[type] = None, type_id: int = 0) -> Optional[_Deserializer]:
        if mtype is None and type_id <= 0:
            raise ValueError("Either mtype or a positive ID must be specified")
        result = None
        if type_id > 0:
            result = self._id_deserializer.get(type_id)

        if result is None and mtype is not None:
            result = self._typed_deserializer.get(mtype)
        return result


_STANDARD_REGISTRY = DeserializerRegistry()


def get_deserializer(
    *, mtype: Optional[type] = None, type_id: int = 0, registry: Optional[DeserializerRegistry] = None
) -> Optional[_Deserializer]:
    if registry is None:
        registry = _STANDARD_REGISTRY
    return registry.get(mtype=mtype, type_id=type_id)


def deserializer(*, type_id: int, registry: Optional[DeserializerRegistry] = None):  # type: ignore
    if type_id < _MAX_BUILT_IN_ID:
        raise ValueError("Custom aggregator ID must be equal or greater than 100")
    if registry is None:
        registry = _STANDARD_REGISTRY

    return _decorate_func(
        key=type_id, name=f"custom.{type_id}", wrapper_dict=registry._id_deserializer, clazz=_Deserializer
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['whylogs',
 'whylogs.api',
 'whylogs.api.logger',
 'whylogs.api.pyspark.experimental',
 'whylogs.api.reader',
 'whylogs.api.store',
 'whylogs.api.usage_stats',
 'whylogs.api.writer',
 'whylogs.core',
 'whylogs.core.constraints',
 'whylogs.core.constraints.factories',
 'whylogs.core.metrics',
 'whylogs.core.model_performance_metrics',
 'whylogs.core.proto',
 'whylogs.core.proto.v0',
 'whylogs.core.utils',
 'whylogs.core.validators',
 'whylogs.core.view',
 'whylogs.datasets',
 'whylogs.datasets.descr',
 'whylogs.extras',
 'whylogs.migration',
 'whylogs.viz',
 'whylogs.viz.enums',
 'whylogs.viz.extensions',
 'whylogs.viz.extensions.reports',
 'whylogs.viz.utils']

package_data = \
{'': ['*'],
 'whylogs.viz': ['html/*',
                 'html/css/*',
                 'html/fonts/*',
                 'html/images/*',
                 'html/js/*',
                 'html/templates/*']}

install_requires = \
['protobuf>=3.15.5', 'typing-extensions', 'whylogs-sketching>=3.4.1.dev3']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=0.23'],
 'datasets': ['pandas'],
 'docs': ['sphinx',
          'sphinx-autoapi',
          'sphinx-copybutton>=0.5.0,<0.6.0',
          'myst-parser[sphinx]>=0.17.2,<0.18.0',
          'furo>=2022.3.4,<2023.0.0',
          'sphinx-autobuild>=2021.3.14,<2022.0.0',
          'sphinxext-opengraph>=0.6.3,<0.7.0',
          'sphinx-inline-tabs',
          'ipython_genutils>=0.2.0,<0.3.0',
          'nbsphinx>=0.8.9,<0.9.0',
          'nbconvert>=7.0.0,<8.0.0'],
 'image': ['Pillow>=9.2.0,<10.0.0'],
 'mlflow': ['mlflow-skinny>=1.26.1,<2.0.0'],
 's3': ['boto3>=1.22.13,<2.0.0'],
 'spark': ['pyarrow>=8.0.0,<9.0.0', 'pyspark>=3.0.0,<4.0.0'],
 'viz': ['pybars3>=0.9,<0.10', 'ipython'],
 'viz:python_version < "3.11"': ['scipy>=1.5'],
 'whylabs': ['whylabs-client>=0.3.0,<0.4.0']}

setup_kwargs = {
    'name': 'whylogs',
    'version': '1.1.0rc3',
    'description': 'Profile and monitor your ML data pipeline end-to-end',
    'long_description': '<img src="https://static.scarf.sh/a.png?x-pxid=bc3c57b0-9a65-49fe-b8ea-f711c4d35b82" /><p align="center">\n<img src="https://i.imgur.com/nv33goV.png" width="35%"/>\n</br>\n\n<h1 align="center">The open standard for data logging\n\n </h1>\n  <h3 align="center">\n   <a href="https://whylogs.readthedocs.io/"><b>Documentation</b></a> &bull;\n   <a href="https://bit.ly/whylogsslack"><b>Slack Community</b></a> &bull;\n   <a href="https://github.com/whylabs/whylogs#python-quickstart"><b>Python Quickstart</b></a>\n </h3>\n\n<p align="center">\n<a href="https://github.com/whylabs/whylogs-python/blob/mainline/LICENSE" target="_blank">\n    <img src="http://img.shields.io/:license-Apache%202-blue.svg" alt="License">\n</a>\n<a href="https://badge.fury.io/py/whylogs" target="_blank">\n    <img src="https://badge.fury.io/py/whylogs.svg" alt="PyPi Version">\n</a>\n<a href="https://github.com/python/black" target="_blank">\n    <img src="https://img.shields.io/badge/code%20style-black-000000.svg" alt="Code style: black">\n</a>\n<a href="https://pepy.tech/project/whylogs" target="_blank">\n    <img src="https://pepy.tech/badge/whylogs" alt="PyPi Downloads">\n</a>\n<a href="bit.ly/whylogs" target="_blank">\n    <img src="https://github.com/whylabs/whylogs-python/workflows/whylogs%20CI/badge.svg" alt="CI">\n</a>\n<a href="https://codeclimate.com/github/whylabs/whylogs-python/maintainability" target="_blank">\n    <img src="https://api.codeclimate.com/v1/badges/442f6ca3dca1e583a488/maintainability" alt="Maintainability">\n</a>\n</p>\n\n## What is whylogs\n\nwhylogs is an open source library for logging any kind of data. With whylogs, users are able to generate summaries of their datasets (called _whylogs profiles_) which they can use to:\n\n1. Track changes in their dataset\n2. Create _data constraints_ to know whether their data looks the way it should\n3. Quickly visualize key summary statistics about their datasets\n\nThese three functionalities enable a variety of use cases for data scientists, machine learning engineers, and data engineers:\n\n- Detect data drift in model input features\n- Detect training-serving skew, concept drift, and model performance degradation\n- Validate data quality in model inputs or in a data pipeline\n- Perform exploratory data analysis of massive datasets\n- Track data distributions & data quality for ML experiments\n- Enable data auditing and governance across the organization\n- Standardize data documentation practices across the organization\n- And more\n\nwhylogs can be run in Python or [Apache Spark](https://docs.whylabs.ai/docs/spark-integration) (both PySpark and Scala) environments on a variety of [data types](#data-types). We [integrate](#integrations) with lots of other tools including Pandas, [AWS Sagemaker](https://aws.amazon.com/blogs/startups/preventing-amazon-sagemaker-model-degradation-with-whylabs/), [MLflow](https://docs.whylabs.ai/docs/mlflow-integration), [Flask](https://whylabs.ai/blog/posts/deploy-and-monitor-your-ml-application-with-flask-and-whylabs), [Ray](https://docs.whylabs.ai/docs/ray-integration), [RAPIDS](https://whylabs.ai/blog/posts/monitoring-high-performance-machine-learning-models-with-rapids-and-whylogs), [Apache Kafka](https://docs.whylabs.ai/docs/kafka-integration), and more.\n\nIf you have any questions, comments, or just want to hang out with us, please join [our Slack Community](https://bit.ly/rsqrd-slack). In addition to joining the Slack Community, you can also help this project by giving us a ⭐ in the upper right corner of this page.\n\n## Python Quickstart<a name="python-quickstart" />\n\nInstalling whylogs using the pip package manager is as easy as running `pip install whylogs` in your terminal.\n\nFrom here, you can quickly log a dataset:\n\n```python\nimport whylogs as why\nimport pandas as pd\n\n#dataframe\ndf = pd.read_csv("path/to/file.csv")\nresults = why.log(df)\n```\n\nAnd voila, you now have a whylogs profile. To learn more about about a whylogs profile is and what you can do with it, read on.\n\n## Table of Contents\n\n- [whylogs Profiles](#whylogs-profiles)\n- [Data Constraints](#data-constraints)\n- [Profile Visualization](#profile-visualization)\n- [Integrations](#integrations)\n- [Supported Data Types](#data-types)\n- [Examples](#examples)\n- [Usage Statistics](#usage-statistics)\n- [Community](#community)\n- [Contribute](#contribute)\n\n## whylogs Profiles<a name="whylogs-profiles" />\n\n### What are profiles\n\nwhylogs profiles are the core of the whylogs library. They capture key statistical properties of data, such as the distribution (far beyond simple mean, median, and standard deviation measures), the number of missing values, and a wide range of configurable custom metrics. By capturing these summary statistics, we are able to accurately represent the data and enable all of the use cases described in the introduction.\n\nwhylogs profiles have three properties that make them ideal for data logging: they are **efficient**, **customizable**, and **mergeable**.\n\n<br />\n\n<img align="left" src="https://user-images.githubusercontent.com/7946482/171064257-26bf727e-3480-4ec3-9c9d-5d8a79567bca.png">\n\n**Efficient**: whylogs profiles efficiently describe the dataset that they represent. This high fidelity representation of datasets is what enables whylogs profiles to be effective snapshots of the data. They are better at capturing the characteristics of a dataset than a sample would be—as discussed in our [Data Logging: Sampling versus Profiling](https://whylabs.ai/blog/posts/data-logging-sampling-versus-profiling) blog post—and are very compact.\n\n<br />\n\n<img align="left" src="https://user-images.githubusercontent.com/7946482/171064575-72ee0f76-7365-4fd1-9cab-4debb673baa8.png">\n\n**Customizable**: The statistics that whylogs profiles collect are easily configured and customizable. This is useful because different data types and use cases require different metrics, and whylogs users need to be able to easily define custom trackers for those metrics. It’s the customizability of whylogs that enables our text, image, and other complex data trackers.\n\n<br />\n\n<img align="left" src="https://user-images.githubusercontent.com/7946482/171064525-2d314534-6cdb-4c07-9d9f-5c74d5c03029.png">\n\n**Mergeable**: One of the most powerful features of whylogs profiles is their mergeability. Mergeability means that whylogs profiles can be combined together to form new profiles which represent the aggregate of their constituent profiles. This enables logging for distributed and streaming systems, and allows users to view aggregated data across any time granularity.\n\n<br />\n\n### How do you generate profiles\n\nOnce whylogs is installed, it\'s easy to generate profiles in both Python and Java environments.\n\nTo generate a profile from a Pandas dataframe in Python, simply run:\n\n```python\nimport whylogs as why\nimport pandas as pd\n\n#dataframe\ndf = pd.read_csv("path/to/file.csv")\nresults = why.log(df)\n```\n\n<!---\nFor images, replace `df` with `image="path/to/image.png"`. Similarly, you can profile Python dicts by replacing the dataframe within the `log()` function with a Python `dict` object.\n--->\n\n### What can you do with profiles\n\nOnce you’ve generated whylogs profiles, a few things can be done with them:\n\nIn your local Python environment, you can set data constraints or visualize your profiles. Setting data constraints on your profiles allows you to get notified when your data don’t match your expectations, allowing you to do data unit testing and some baseline data monitoring. With the Profile Visualizer, you can visually explore your data, allowing you to understand it and ensure that your ML models are ready for production.\n\nIn addition, you can send whylogs profiles to the SaaS ML monitoring and AI observability platform [WhyLabs](https://whylabs.ai). With WhyLabs, you can automatically set up monitoring for your machine learning models, getting notified on both data quality and data change issues (such as data drift). If you’re interested in trying out WhyLabs, check out the always free [Starter edition](https://whylabs.ai/free), which allows you to experience the entire platform’s capabilities with no credit card required.\n\n## Data Constraints<a name="data-constraints" />\n\nConstraints are a powerful feature built on top of whylogs profiles that enable you to quickly and easily validate that your data looks the way that it should. There are numerous types of constraints that you can set on your data (that numerical data will always fall within a certain range, that text data will always be in a JSON format, etc) and, if your dataset fails to satisfy a constraint, you can fail your unit tests or your CI/CD pipeline.\n\nA simple example of setting and testing a constraint is:\n\n```python\nimport whylogs as why\nfrom whylogs.core.constraints import Constraints, ConstraintsBuilder, MetricsSelector, MetricConstraint\n\nprofile_view = why.log(df).view()\nbuilder = ConstraintsBuilder(profile_view)\n\nbuilder.add_constraint(MetricConstraint(\n    name="col_name >= 0",\n    condition=lambda x: x.min >= 0,\n    metric_selector=MetricsSelector(metric_name=\'distribution\', column_name=\'col_name\')\n))\nconstraints: Constraints = builder.build()\nconstraints.report()\n```\n\nTo learn more about constraints, check out: the [Constraints Example](https://bit.ly/whylogsconstraintsexample).\n\n## Profile Visualization<a name="profile-visualization" />\n\nIn addition to being able to automatically get notified about potential issues in data, it’s also useful to be able to inspect your data manually. With the profile visualizer, you can generate interactive reports about your profiles (either a single profile or comparing profiles against each other) directly in your Jupyter notebook environment. This enables exploratory data analysis, data drift detection, and data observability.\n\nTo access the profile visualizer, install the `[viz]` module of whylogs by running `pip install whylogs[viz]` in your terminal. One type of profile visualization that we can create is a drift report; here\'s a simple example of how to analyze the drift between two profiles:\n\n```python\nimport whylogs as why\n\nfrom whylogs.viz import NotebookProfileVisualizer\n\nresult = why.log(pandas=df_target)\nprof_view = result.view()\n\nresult_ref = why.log(pandas=df_reference)\nprof_view_ref = result_ref.view()\n\nvisualization = NotebookProfileVisualizer()\nvisualization.set_profiles(target_profile_view=prof_view, reference_profile_view=prof_view_ref)\n\nvisualization.summary_drift_report()\n```\n\n![image](https://user-images.githubusercontent.com/7946482/169669536-a25cce95-acde-4637-b7b9-c2a685f0bc3f.png)\n\nTo learn more about visualizing your profiles, check out: the [Visualizer Example](https://bit.ly/whylogsvisualizerexample)\n\n## Data Types<a name="data-types" />\n\nwhylogs supports both structured and unstructured data, specifically:\n\n| Data type        | Features | Notebook Example                                                                                                                                                |\n| ---------------- | -------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------- |\n| Tabular Data     | ✅       | [Getting started with structured data](https://github.com/whylabs/whylogs/blob/mainline/python/examples/basic/Getting_Started.ipynb)                            |\n| Image Data       | ✅       | [Getting started with images](https://github.com/whylabs/whylogs/blob/maintenance/0.7.x/examples/logging_images.ipynb)                                          |\n| Text Data        | ✅       | [String Features](https://github.com/whylabs/whylogs/blob/maintenance/0.7.x/examples/String_Features.ipynb)                                                     |\n| Embeddings       | 🛠        |                                                                                                                                                                 |\n| Other Data Types | ✋       | Do you have a request for a data type that you don’t see listed here? Raise an issue or join our Slack community and make a request! We’re always happy to help |\n\n## Integrations\n\n![current integration](https://user-images.githubusercontent.com/7946482/171062942-01c420f2-7768-4b7c-88b5-e3f291e1b7d8.png)\n| Integration | Features | Resources |\n| --- | --- | --- |\n| Spark | Run whylogs in Apache Spark environment| <ul><li>[Code Example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/integrations/Pyspark_Profiling.ipynb)</li></ul> |\n| Pandas | Log and monitor any pandas dataframe | <ul><li>[Notebook Example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/basic/Getting_Started.ipynb)</li><li>[whylogs: Embrace Data Logging](https://whylabs.ai/blog/posts/whylogs-embrace-data-logging)</li></ul> |\n| MLflow | Enhance MLflow metrics with whylogs: | <ul><li>[Notebook Example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/integrations/Mlflow_Logging.ipynb)</li><li>[Streamlining data monitoring with whylogs and MLflow](https://whylabs.ai/blog/posts/on-model-lifecycle-and-monitoring)</li></ul> |\n| Java | Run whylogs in Java environment| <ul><li>[Notebook Example](https://github.com/whylabs/whylogs-examples/blob/mainline/java/demo1/src/main/java/com/whylogs/examples/WhyLogsDemo.java)</li></ul> |\n| Docker | Run whylogs as in Docker | <ul><li>[Rest Container](https://docs.whylabs.ai/docs/integrations-rest-container)</li></ul>|\n| AWS S3 | Store whylogs profiles in S3 | <ul><li>[S3 example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/integrations/writers/Writing_Profiles.ipynb)</li></ul>\n| Flask | Integrate whylogs with your Flask Application| <ul><li>[Code Example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/integrations/flask_streaming/flask_with_whylogs.ipynb)</li></ul> |\n| Feast | Log features from your Feature Store with feast and whylogs| <ul><li>[Code Example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/integrations/Feature_Stores_and_whylogs.ipynb)</li></ul> |\n| BigQuery | Profile data queried from a Google BigQuery table| <ul><li>[Code Example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/integrations/BigQuery_Example.ipynb)</li></ul> |\n| WhyLabs | Monitor your profiles continuously with the WhyLabs Observability Platform| <ul><li>[Code Example](https://github.com/whylabs/whylogs/blob/mainline/python/examples/integrations/writers/Writing_to_WhyLabs.ipynb)</li></ul> |\n\n<!--| Kafka | Log and monitor Kafka topics with whylogs| <ul><li>[Notebook Example](https://github.com/whylabs/whylogs-examples/blob/mainline/python/Kafka.ipynb)</li><li> [Integrating whylogs into your Kafka ML Pipeline](https://whylabs.ai/blog/posts/integrating-whylogs-into-your-kafka-ml-pipeline) </li></ul>|-->\n<!--| Github actions | Unit test data with whylogs and github actions| <ul><li>[Notebook Example](https://github.com/whylabs/whylogs-examples/tree/mainline/github-actions)</li></ul> |-->\n<!--| RAPIDS | Use whylogs in RAPIDS environment | <ul><li>[Notebook Example](https://github.com/whylabs/whylogs-examples/blob/mainline/python/RAPIDS%20GPU%20Integration%20Example.ipynb)</li><li>[Monitoring High-Performance Machine Learning Models with RAPIDS and whylogs](https://whylabs.ai/blog/posts/monitoring-high-performance-machine-learning-models-with-rapids-and-whylogs)</li></ul> |-->\n\n## Examples\n\nFor a full set of our examples, please check out the [examples folder](https://github.com/whylabs/whylogs/tree/mainline/python/examples).\n\n## Usage Statistics<a name="whylogs-profiles" />\n\nStarting with whylogs v1.0.0, whylogs by default collects anonymous information about a user’s environment. These usage statistics do not include any information about the user or the data that they are profiling, only the environment that the user in which the user is running whylogs.\n\nTo read more about what usage statistics whylogs collects, check out the relevant [documentation](https://docs.whylabs.ai/docs/usage-statistics/).\n\nTo turn off Usage Statistics, simply set the `WHYLOGS_NO_ANALYTICS` environment variable to True, like so:\n\n```python\nimport os\nos.environ[\'WHYLOGS_NO_ANALYTICS\']=\'True\'\n```\n\n## Community\n\nIf you have any questions, comments, or just want to hang out with us, please join [our Slack channel](http://join.slack.whylabs.ai/).\n\n## Contribute\n\n### How to Contribute\n\nWe welcome contributions to whylogs. Please see our [contribution guide](https://github.com/whylabs/whylogs/blob/mainline/.github/CONTRIBUTING.md) and our [development guide](https://github.com/whylabs/whylogs/blob/mainline/.github/DEVELOPMENT.md) for details.\n\n### Contributors\n\n<a href="https://github.com/whylabs/whylogs/graphs/contributors">\n  <img src="https://contrib.rocks/image?repo=whylabs/whylogs" />\n</a>\n\nMade with [contrib.rocks](https://contrib.rocks).\n',
    'author': 'WhyLabs.ai',
    'author_email': 'support@whylabs.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://docs.whylabs.ai',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7.1,<4',
}


setup(**setup_kwargs)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.struct_pb2
import typing
import typing_extensions
import v0_messages_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class UniqueCountSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ESTIMATE_FIELD_NUMBER: builtins.int
    UPPER_FIELD_NUMBER: builtins.int
    LOWER_FIELD_NUMBER: builtins.int
    estimate: builtins.float
    upper: builtins.float
    lower: builtins.float
    def __init__(self,
        *,
        estimate: builtins.float = ...,
        upper: builtins.float = ...,
        lower: builtins.float = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["estimate",b"estimate","lower",b"lower","upper",b"upper"]) -> None: ...
global___UniqueCountSummary = UniqueCountSummary

class FrequentStringsSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class FrequentItem(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        VALUE_FIELD_NUMBER: builtins.int
        ESTIMATE_FIELD_NUMBER: builtins.int
        value: typing.Text
        estimate: builtins.float
        def __init__(self,
            *,
            value: typing.Text = ...,
            estimate: builtins.float = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["estimate",b"estimate","value",b"value"]) -> None: ...

    ITEMS_FIELD_NUMBER: builtins.int
    @property
    def items(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FrequentStringsSummary.FrequentItem]: ...
    def __init__(self,
        *,
        items: typing.Optional[typing.Iterable[global___FrequentStringsSummary.FrequentItem]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["items",b"items"]) -> None: ...
global___FrequentStringsSummary = FrequentStringsSummary

class FrequentNumbersSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class FrequentDoubleItem(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        ESTIMATE_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        RANK_FIELD_NUMBER: builtins.int
        estimate: builtins.int
        value: builtins.float
        rank: builtins.int
        def __init__(self,
            *,
            estimate: builtins.int = ...,
            value: builtins.float = ...,
            rank: builtins.int = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["estimate",b"estimate","rank",b"rank","value",b"value"]) -> None: ...

    class FrequentLongItem(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        ESTIMATE_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        RANK_FIELD_NUMBER: builtins.int
        estimate: builtins.int
        value: builtins.int
        rank: builtins.int
        def __init__(self,
            *,
            estimate: builtins.int = ...,
            value: builtins.int = ...,
            rank: builtins.int = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["estimate",b"estimate","rank",b"rank","value",b"value"]) -> None: ...

    DOUBLES_FIELD_NUMBER: builtins.int
    LONGS_FIELD_NUMBER: builtins.int
    @property
    def doubles(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FrequentNumbersSummary.FrequentDoubleItem]: ...
    @property
    def longs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FrequentNumbersSummary.FrequentLongItem]: ...
    def __init__(self,
        *,
        doubles: typing.Optional[typing.Iterable[global___FrequentNumbersSummary.FrequentDoubleItem]] = ...,
        longs: typing.Optional[typing.Iterable[global___FrequentNumbersSummary.FrequentLongItem]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["doubles",b"doubles","longs",b"longs"]) -> None: ...
global___FrequentNumbersSummary = FrequentNumbersSummary

class FrequentItemsSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class FrequentItem(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        ESTIMATE_FIELD_NUMBER: builtins.int
        JSON_VALUE_FIELD_NUMBER: builtins.int
        estimate: builtins.int
        json_value: typing.Text
        def __init__(self,
            *,
            estimate: builtins.int = ...,
            json_value: typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["estimate",b"estimate","json_value",b"json_value"]) -> None: ...

    ITEMS_FIELD_NUMBER: builtins.int
    @property
    def items(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FrequentItemsSummary.FrequentItem]: ...
    def __init__(self,
        *,
        items: typing.Optional[typing.Iterable[global___FrequentItemsSummary.FrequentItem]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["items",b"items"]) -> None: ...
global___FrequentItemsSummary = FrequentItemsSummary

class CharPosSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class CharPosMapEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        @property
        def value(self) -> global___NumberSummary: ...
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: typing.Optional[global___NumberSummary] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value",b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    CHARACTER_LIST_FIELD_NUMBER: builtins.int
    CHAR_POS_MAP_FIELD_NUMBER: builtins.int
    character_list: typing.Text
    @property
    def char_pos_map(self) -> google.protobuf.internal.containers.MessageMap[typing.Text, global___NumberSummary]: ...
    def __init__(self,
        *,
        character_list: typing.Text = ...,
        char_pos_map: typing.Optional[typing.Mapping[typing.Text, global___NumberSummary]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["char_pos_map",b"char_pos_map","character_list",b"character_list"]) -> None: ...
global___CharPosSummary = CharPosSummary

class StringsSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    UNIQUE_COUNT_FIELD_NUMBER: builtins.int
    FREQUENT_FIELD_NUMBER: builtins.int
    LENGTH_FIELD_NUMBER: builtins.int
    TOKEN_LENGTH_FIELD_NUMBER: builtins.int
    CHAR_POS_TRACKER_FIELD_NUMBER: builtins.int
    @property
    def unique_count(self) -> global___UniqueCountSummary: ...
    @property
    def frequent(self) -> global___FrequentStringsSummary: ...
    @property
    def length(self) -> global___NumberSummary: ...
    @property
    def token_length(self) -> global___NumberSummary: ...
    @property
    def char_pos_tracker(self) -> global___CharPosSummary: ...
    def __init__(self,
        *,
        unique_count: typing.Optional[global___UniqueCountSummary] = ...,
        frequent: typing.Optional[global___FrequentStringsSummary] = ...,
        length: typing.Optional[global___NumberSummary] = ...,
        token_length: typing.Optional[global___NumberSummary] = ...,
        char_pos_tracker: typing.Optional[global___CharPosSummary] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["char_pos_tracker",b"char_pos_tracker","frequent",b"frequent","length",b"length","token_length",b"token_length","unique_count",b"unique_count"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["char_pos_tracker",b"char_pos_tracker","frequent",b"frequent","length",b"length","token_length",b"token_length","unique_count",b"unique_count"]) -> None: ...
global___StringsSummary = StringsSummary

class SchemaSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class TypeCountsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        value: builtins.int
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: builtins.int = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    INFERRED_TYPE_FIELD_NUMBER: builtins.int
    TYPE_COUNTS_FIELD_NUMBER: builtins.int
    @property
    def inferred_type(self) -> v0_messages_pb2.InferredType: ...
    @property
    def type_counts(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, builtins.int]: ...
    def __init__(self,
        *,
        inferred_type: typing.Optional[v0_messages_pb2.InferredType] = ...,
        type_counts: typing.Optional[typing.Mapping[typing.Text, builtins.int]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["inferred_type",b"inferred_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["inferred_type",b"inferred_type","type_counts",b"type_counts"]) -> None: ...
global___SchemaSummary = SchemaSummary

class HistogramSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    START_FIELD_NUMBER: builtins.int
    END_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    COUNTS_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    MIN_FIELD_NUMBER: builtins.int
    BINS_FIELD_NUMBER: builtins.int
    N_FIELD_NUMBER: builtins.int
    start: builtins.float
    end: builtins.float
    width: builtins.float
    @property
    def counts(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    max: builtins.float
    min: builtins.float
    @property
    def bins(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    n: builtins.int
    def __init__(self,
        *,
        start: builtins.float = ...,
        end: builtins.float = ...,
        width: builtins.float = ...,
        counts: typing.Optional[typing.Iterable[builtins.int]] = ...,
        max: builtins.float = ...,
        min: builtins.float = ...,
        bins: typing.Optional[typing.Iterable[builtins.float]] = ...,
        n: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["bins",b"bins","counts",b"counts","end",b"end","max",b"max","min",b"min","n",b"n","start",b"start","width",b"width"]) -> None: ...
global___HistogramSummary = HistogramSummary

class QuantileSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    QUANTILES_FIELD_NUMBER: builtins.int
    QUANTILE_VALUES_FIELD_NUMBER: builtins.int
    @property
    def quantiles(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def quantile_values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(self,
        *,
        quantiles: typing.Optional[typing.Iterable[builtins.float]] = ...,
        quantile_values: typing.Optional[typing.Iterable[builtins.float]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["quantile_values",b"quantile_values","quantiles",b"quantiles"]) -> None: ...
global___QuantileSummary = QuantileSummary

class NumberSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COUNT_FIELD_NUMBER: builtins.int
    MIN_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    MEAN_FIELD_NUMBER: builtins.int
    STDDEV_FIELD_NUMBER: builtins.int
    HISTOGRAM_FIELD_NUMBER: builtins.int
    UNIQUE_COUNT_FIELD_NUMBER: builtins.int
    QUANTILES_FIELD_NUMBER: builtins.int
    FREQUENT_NUMBERS_FIELD_NUMBER: builtins.int
    IS_DISCRETE_FIELD_NUMBER: builtins.int
    count: builtins.int
    min: builtins.float
    max: builtins.float
    mean: builtins.float
    stddev: builtins.float
    @property
    def histogram(self) -> global___HistogramSummary: ...
    @property
    def unique_count(self) -> global___UniqueCountSummary: ...
    @property
    def quantiles(self) -> global___QuantileSummary: ...
    @property
    def frequent_numbers(self) -> global___FrequentNumbersSummary: ...
    is_discrete: builtins.bool
    def __init__(self,
        *,
        count: builtins.int = ...,
        min: builtins.float = ...,
        max: builtins.float = ...,
        mean: builtins.float = ...,
        stddev: builtins.float = ...,
        histogram: typing.Optional[global___HistogramSummary] = ...,
        unique_count: typing.Optional[global___UniqueCountSummary] = ...,
        quantiles: typing.Optional[global___QuantileSummary] = ...,
        frequent_numbers: typing.Optional[global___FrequentNumbersSummary] = ...,
        is_discrete: builtins.bool = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["frequent_numbers",b"frequent_numbers","histogram",b"histogram","quantiles",b"quantiles","unique_count",b"unique_count"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["count",b"count","frequent_numbers",b"frequent_numbers","histogram",b"histogram","is_discrete",b"is_discrete","max",b"max","mean",b"mean","min",b"min","quantiles",b"quantiles","stddev",b"stddev","unique_count",b"unique_count"]) -> None: ...
global___NumberSummary = NumberSummary

class ColumnSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COUNTERS_FIELD_NUMBER: builtins.int
    SCHEMA_FIELD_NUMBER: builtins.int
    NUMBER_SUMMARY_FIELD_NUMBER: builtins.int
    STRING_SUMMARY_FIELD_NUMBER: builtins.int
    FREQUENT_ITEMS_FIELD_NUMBER: builtins.int
    UNIQUE_COUNT_FIELD_NUMBER: builtins.int
    @property
    def counters(self) -> v0_messages_pb2.CountersV0: ...
    @property
    def schema(self) -> global___SchemaSummary: ...
    @property
    def number_summary(self) -> global___NumberSummary: ...
    @property
    def string_summary(self) -> global___StringsSummary: ...
    @property
    def frequent_items(self) -> global___FrequentItemsSummary: ...
    @property
    def unique_count(self) -> global___UniqueCountSummary: ...
    def __init__(self,
        *,
        counters: typing.Optional[v0_messages_pb2.CountersV0] = ...,
        schema: typing.Optional[global___SchemaSummary] = ...,
        number_summary: typing.Optional[global___NumberSummary] = ...,
        string_summary: typing.Optional[global___StringsSummary] = ...,
        frequent_items: typing.Optional[global___FrequentItemsSummary] = ...,
        unique_count: typing.Optional[global___UniqueCountSummary] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["counters",b"counters","frequent_items",b"frequent_items","number_summary",b"number_summary","schema",b"schema","string_summary",b"string_summary","unique_count",b"unique_count"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["counters",b"counters","frequent_items",b"frequent_items","number_summary",b"number_summary","schema",b"schema","string_summary",b"string_summary","unique_count",b"unique_count"]) -> None: ...
global___ColumnSummary = ColumnSummary

class DatasetSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        @property
        def value(self) -> global___ColumnSummary: ...
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: typing.Optional[global___ColumnSummary] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value",b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    PROPERTIES_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    MODEL_FIELD_NUMBER: builtins.int
    @property
    def properties(self) -> v0_messages_pb2.DatasetPropertiesV0: ...
    @property
    def columns(self) -> google.protobuf.internal.containers.MessageMap[typing.Text, global___ColumnSummary]: ...
    @property
    def model(self) -> global___ModelSummary: ...
    def __init__(self,
        *,
        properties: typing.Optional[v0_messages_pb2.DatasetPropertiesV0] = ...,
        columns: typing.Optional[typing.Mapping[typing.Text, global___ColumnSummary]] = ...,
        model: typing.Optional[global___ModelSummary] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["model",b"model","properties",b"properties"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns",b"columns","model",b"model","properties",b"properties"]) -> None: ...
global___DatasetSummary = DatasetSummary

class ModelSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    METRICS_FIELD_NUMBER: builtins.int
    @property
    def metrics(self) -> global___MetricsSummary: ...
    def __init__(self,
        *,
        metrics: typing.Optional[global___MetricsSummary] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["metrics",b"metrics"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["metrics",b"metrics"]) -> None: ...
global___ModelSummary = ModelSummary

class MetricsSummary(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MODEL_TYPE_FIELD_NUMBER: builtins.int
    ROC_FPR_TPR_FIELD_NUMBER: builtins.int
    RECALL_PREC_FIELD_NUMBER: builtins.int
    CONFUSION_MATRIX_FIELD_NUMBER: builtins.int
    model_type: v0_messages_pb2.ModelType.ValueType
    @property
    def roc_fpr_tpr(self) -> global___ROCCurve: ...
    @property
    def recall_prec(self) -> global___RecallCurve: ...
    @property
    def confusion_matrix(self) -> global___ConfusionMatrix: ...
    def __init__(self,
        *,
        model_type: v0_messages_pb2.ModelType.ValueType = ...,
        roc_fpr_tpr: typing.Optional[global___ROCCurve] = ...,
        recall_prec: typing.Optional[global___RecallCurve] = ...,
        confusion_matrix: typing.Optional[global___ConfusionMatrix] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["confusion_matrix",b"confusion_matrix","recall_prec",b"recall_prec","roc_fpr_tpr",b"roc_fpr_tpr"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["confusion_matrix",b"confusion_matrix","model_type",b"model_type","recall_prec",b"recall_prec","roc_fpr_tpr",b"roc_fpr_tpr"]) -> None: ...
global___MetricsSummary = MetricsSummary

class ConfusionMatrix(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LABELS_FIELD_NUMBER: builtins.int
    TARGET_FIELD_FIELD_NUMBER: builtins.int
    PREDICTIONS_FIELD_FIELD_NUMBER: builtins.int
    SCORE_FIELD_FIELD_NUMBER: builtins.int
    COUNTS_FIELD_NUMBER: builtins.int
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    target_field: typing.Text
    predictions_field: typing.Text
    score_field: typing.Text
    @property
    def counts(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.struct_pb2.ListValue]:
        """e.g. [[33, 6], [11, 27]]"""
        pass
    def __init__(self,
        *,
        labels: typing.Optional[typing.Iterable[typing.Text]] = ...,
        target_field: typing.Text = ...,
        predictions_field: typing.Text = ...,
        score_field: typing.Text = ...,
        counts: typing.Optional[typing.Iterable[google.protobuf.struct_pb2.ListValue]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["counts",b"counts","labels",b"labels","predictions_field",b"predictions_field","score_field",b"score_field","target_field",b"target_field"]) -> None: ...
global___ConfusionMatrix = ConfusionMatrix

class ROCCurve(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.struct_pb2.ListValue]:
        """e.g.   "values": [ [1, 0.42857],  [1, 0.42857], ... ]"""
        pass
    def __init__(self,
        *,
        values: typing.Optional[typing.Iterable[google.protobuf.struct_pb2.ListValue]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["values",b"values"]) -> None: ...
global___ROCCurve = ROCCurve

class RecallCurve(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.struct_pb2.ListValue]:
        """e.g.   "values": [ [1, 1],  [1, 1], ... ]"""
        pass
    def __init__(self,
        *,
        values: typing.Optional[typing.Iterable[google.protobuf.struct_pb2.ListValue]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["values",b"values"]) -> None: ...
global___RecallCurve = RecallCurve

class DatasetSummaries(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class ProfilesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        @property
        def value(self) -> global___DatasetSummary: ...
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: typing.Optional[global___DatasetSummary] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value",b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    PROFILES_FIELD_NUMBER: builtins.int
    @property
    def profiles(self) -> google.protobuf.internal.containers.MessageMap[typing.Text, global___DatasetSummary]: ...
    def __init__(self,
        *,
        profiles: typing.Optional[typing.Mapping[typing.Text, global___DatasetSummary]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["profiles",b"profiles"]) -> None: ...
global___DatasetSummaries = DatasetSummaries



from typing import Any, Dict, Mapping, Optional, Tuple

from whylogs.core.stubs import pd


def _pandas_or_dict(
    obj: Any, pandas: Optional[pd.DataFrame] = None, row: Optional[Mapping[str, Any]] = None
) -> Tuple[Optional[pd.DataFrame], Optional[Mapping[str, Any]]]:
    if obj is not None:
        if pandas is not None:
            raise ValueError("Cannot pass both obj and pandas params")
        if row is not None:
            raise ValueError("Cannot pass both obj and row params")

        if pd.DataFrame is not None and isinstance(obj, pd.DataFrame):
            pandas = obj
        elif isinstance(obj, (dict, Dict, Mapping)):
            row = obj

    if pandas is not None and row is not None:
        raise ValueError("Cannot pass both pandas and row params")

    return (pandas, row)



from typing import Any

from whylogs.api.reader.reader import Reader


class Readers:
    @staticmethod
    def get(name: str, **kwargs: Any) -> "Reader":
        if name == "local":
            from whylogs.api.reader.local import LocalReader

            return LocalReader(**kwargs)
        elif name == "s3":
            from whylogs.api.reader.s3 import S3Reader

            return S3Reader(**kwargs)

        raise ValueError(f"Unrecognized reader: {name}")



import logging
from itertools import chain
from typing import Dict, List, Union

import whylogs as why
from whylogs.api.logger.result_set import ResultSet
from whylogs.core.datatypes import DataType
from whylogs.core.metrics.compound_metric import CompoundMetric
from whylogs.core.metrics.metrics import (
    DistributionMetric,
    FrequentItemsMetric,
    Metric,
    MetricConfig,
    OperationResult,
)
from whylogs.core.preprocessing import PreprocessedColumn
from whylogs.core.resolvers import Resolver
from whylogs.core.schema import ColumnSchema, DatasetSchema

logger = logging.getLogger(__name__)

try:
    from PIL.Image import Image as ImageType
    from PIL.Image import new as new_image
    from PIL.ImageStat import Stat
    from PIL.TiffImagePlugin import IFDRational
    from PIL.TiffTags import TAGS
except ImportError as e:
    ImageType = None
    logger.warning(str(e))
    logger.warning("Unable to load PIL; install Pillow for image support")

DEFAULT_IMAGE_FEATURES: List[str] = []


_DEFAULT_TAGS_ATTRIBUTES = [
    "ImagePixelWidth",
    "ImagePixelHeight",
    "Colorspace",
]
_IMAGE_HSV_CHANNELS = ["Hue", "Saturation", "Brightness"]
_STATS_PROPERTIES = ["mean", "stddev"]
_DEFAULT_STAT_ATTRIBUTES = [c + "." + s for c in _IMAGE_HSV_CHANNELS for s in _STATS_PROPERTIES]
_METADATA_DEFAULT_ATTRIBUTES = _DEFAULT_STAT_ATTRIBUTES + _DEFAULT_TAGS_ATTRIBUTES


def get_pil_image_statistics(
    img: ImageType, channels: List[str] = _IMAGE_HSV_CHANNELS, image_stats: List[str] = _STATS_PROPERTIES
) -> Dict:
    """
    Compute statistics data for a PIL Image

    Args:
        img (ImageType): PIL Image

    Returns:
        Dict: of metadata
    """

    stats = Stat(img.convert("HSV"))
    metadata = {}
    for index in range(len(channels)):
        for statistic_name in image_stats:
            if hasattr(stats, statistic_name):
                metadata[channels[index] + "." + statistic_name] = getattr(stats, statistic_name)[index]

    return metadata


def get_pil_image_metadata(img: ImageType) -> Dict:
    """
    Grab statistics data from a PIL ImageStats.Stat

    Args:
        img (ImageType): PIL Image

    Returns:
        Dict: of metadata
    """
    metadata = {}
    for k, v in dict(img.getexif()).items():
        try:
            if isinstance(v, IFDRational):
                metadata[TAGS[k]] = "{}".format(v)
            else:
                metadata[TAGS[k]] = v
        except KeyError:
            logger.warning(f"Couldn't read exif tag: {k} skipping.")

    metadata.update(image_based_metadata(img))
    metadata.update(get_pil_image_statistics(img))

    return metadata


def image_based_metadata(img):
    return {
        "ImagePixelWidth": img.width,
        "ImagePixelHeight": img.height,
        "Colorspace": img.mode,
    }


class ImageMetric(CompoundMetric):
    def __init__(self, submetrics: Dict[str, Metric]):
        if ImageType is None:
            logger.error("Install Pillow for image support")
        super(ImageMetric, self).__init__(submetrics)

    @property
    def namespace(self) -> str:
        return "image"

    def columnar_update(self, view: PreprocessedColumn) -> OperationResult:
        submetric_names = self.submetrics.keys()
        count = 0
        for image in list(chain.from_iterable(view.raw_iterator())):
            if isinstance(image, ImageType):
                metadata = get_pil_image_metadata(image)
                for attr in metadata.keys():
                    if attr in submetric_names:
                        self.submetrics[attr].columnar_update(PreprocessedColumn.apply([metadata[attr]]))
                count += 1
        return OperationResult.ok(count)

    @classmethod
    def zero(cls, config: MetricConfig) -> "ImageMetric":
        dummy_image = new_image("HSV", (1, 1))
        metadata = get_pil_image_metadata(dummy_image)
        submetrics: Dict[str, Metric] = dict()
        for tag, value in metadata.items():
            if isinstance(value, int):
                submetrics[tag] = DistributionMetric.zero(config)
            elif isinstance(value, float):
                submetrics[tag] = DistributionMetric.zero(config)
            elif isinstance(value, str) and not config.fi_disabled:
                submetrics[tag] = FrequentItemsMetric.zero(config)

        return ImageMetric(submetrics)


"""
def log_image(
    profiles: Union[List[DatasetProfile],DatasetProfile],
    images: Union[Dict[str, ImageType], ImageType],
    default_column_name: str="image",
) -> None:
    profiles = profiles if isinstance(profiles, list) else [profiles]
    images = images if isinstance(images, dict) else {default_column_name: images}
    for profile in profiles:
        profile.track(row=images)
"""


def log_image(
    images: Union[ImageType, List[ImageType], Dict[str, ImageType]],
    default_column_prefix: str = "image",
) -> ResultSet:
    if isinstance(images, ImageType):
        images = {default_column_prefix: images}
    if isinstance(images, list):
        count = 1
        images = dict()
        for img in images:
            images[f"{default_column_prefix}_{count}"] = img
            count += 1

    if not isinstance(images, dict):
        raise ValueError("log_image must be passed an image, list of images, or dictionary of images")
    for img in images.values():
        if not isinstance(img, ImageType):
            raise ValueError("log_image must be passed an image, list of images, or dictionary of images")

    class ImageResolver(Resolver):
        def resolve(self, name: str, why_type: DataType, column_schema: ColumnSchema) -> Dict[str, Metric]:
            return {ImageMetric.get_namespace(MetricConfig()): ImageMetric.zero(MetricConfig())}

    schema = DatasetSchema(types={key: ImageType for key in images.keys()}, resolvers=ImageResolver())
    return why.log(row=images, schema=schema)



from whylogs.viz.extensions.reports.html_report import HTMLReport


class DistributionChart(HTMLReport):
    pass


class DifferenceDistributionChart(HTMLReport):
    pass


class FeatureStatisticsReport(HTMLReport):
    pass


class DoubleHistogram(HTMLReport):
    pass



from whylogs.viz.utils.descriptive_stats import (
    DescriptiveStatistics,
    _calculate_descriptive_statistics,
)
from whylogs.viz.utils.quantile_stats import (
    QuantileStats,
    _calculate_bins,
    _calculate_quantile_statistics,
)

__all__ = [
    "_calculate_descriptive_statistics",
    "_calculate_quantile_statistics",
    "_calculate_bins",
    "DescriptiveStatistics",
    "QuantileStats",
]



from typing import Any

from whylogs.api.writer.writer import Writer


class Writers:
    @staticmethod
    def get(name: str, **kwargs: Any) -> "Writer":
        if name == "local":
            from whylogs.api.writer.local import LocalWriter

            return LocalWriter(**kwargs)  # type: ignore
        elif name == "whylabs":
            from whylogs.api.writer.whylabs import WhyLabsWriter

            return WhyLabsWriter(**kwargs)  # type: ignore
        elif name == "s3":
            from whylogs.api.writer.s3 import S3Writer

            return S3Writer(**kwargs)  # type: ignore
        elif name == "mlflow":
            from whylogs.api.writer.mlflow import MlflowWriter

            return MlflowWriter(**kwargs)  # type: ignore
        raise ValueError(f"Unrecognized writer: {name}")



from typing import List, Optional, Tuple

from typing_extensions import TypedDict

from whylogs.core.configs import SummaryConfig
from whylogs.core.metrics.metrics import FrequentItem
from whylogs.core.view.column_profile_view import ColumnProfileView


class FrequentItemEstimate(TypedDict):
    value: str
    estimate: float


def get_frequent_items_estimate(frequent_items: List[FrequentItem]) -> List[FrequentItemEstimate]:
    return [{"value": x.value, "estimate": x.est} for x in frequent_items]


class FrequentStats(TypedDict):
    frequent_items: List[FrequentItemEstimate]
    total_count: int
    unique_count: int


def get_frequent_stats(column_view: ColumnProfileView, config: Optional[SummaryConfig]) -> Optional[FrequentStats]:
    if config is None:
        config = SummaryConfig()

    target_fi_metric = column_view.get_metric("frequent_items")
    if target_fi_metric is None:
        return None

    target_frequent_items = target_fi_metric.to_summary_dict(config)["frequent_strings"]
    target_cnt_metric = column_view.get_metric("counts")
    target_count = target_cnt_metric.n.value
    target_card_metric = column_view.get_metric("cardinality")
    target_unique_count = int(target_card_metric.hll.value.get_estimate())

    target_frequent_stats: FrequentStats = {
        "frequent_items": get_frequent_items_estimate(target_frequent_items),
        "total_count": target_count,
        "unique_count": target_unique_count,
    }

    return target_frequent_stats


def frequent_items_from_view(
    column_view: ColumnProfileView, feature_name: str, config: Optional[SummaryConfig] = None
) -> List[FrequentItemEstimate]:
    config = config or SummaryConfig()
    column_frequent_items_metric = column_view.get_metric("frequent_items")
    if not column_frequent_items_metric:
        raise ValueError("Frequent Items Metrics not found for feature {}.".format(feature_name))

    target_frequent_items = column_frequent_items_metric.to_summary_dict(config)["frequent_strings"]
    frequent_items = get_frequent_items_estimate(target_frequent_items)
    return frequent_items


def zero_padding_frequent_items(
    target_feature_items: List[FrequentItemEstimate], reference_feature_items: List[FrequentItemEstimate]
) -> Tuple[List[FrequentItemEstimate], List[FrequentItemEstimate]]:
    """Fills estimate value of item with 0 when such item is present in the other profile but absent in the current profile.
    This is done for both profiles passed.

    Parameters
    ----------
    target_feature_items : List[FrequentItemEstimate]
        A list of frequent items of a given column for target profile
    reference_feature_items : List[FrequentItemEstimate]
        A list of frequent items of a given column for reference profile

    Returns
    -------
    Tuple[List[FrequentItemEstimate], List[FrequentItemEstimate]]
        The same list of items given in the input, but with zero padding for absent items.
    """
    for reference_item in reference_feature_items:
        item_value = reference_item["value"]
        if item_value not in [x["value"] for x in target_feature_items]:
            target_feature_items.append({"value": item_value, "estimate": 0})
    for target_item in target_feature_items:
        item_value = target_item["value"]
        if item_value not in [x["value"] for x in reference_feature_items]:
            reference_feature_items.append({"value": item_value, "estimate": 0})

    return target_feature_items, reference_feature_items



from abc import ABC
from typing import Any, Dict, List, Type, TypeVar

from whylogs.core.configs import SummaryConfig
from whylogs.core.errors import DeserializationError, UnsupportedError
from whylogs.core.metrics import Metric
from whylogs.core.metrics.metrics import _METRIC_DESERIALIZER_REGISTRY, OperationResult
from whylogs.core.preprocessing import PreprocessedColumn
from whylogs.core.proto import MetricComponentMessage, MetricMessage

COMPOUND_METRIC = TypeVar("COMPOUND_METRIC", bound="CompoundMetric")


class CompoundMetric(Metric, ABC):
    """
    CompoundMetric serves as a base class for custom metrics that consist
    of one or more metrics. It is handy when you need to do some
    processing of the logged values and track serveral metrics on
    the results. The sub-metrics must either be a StandardMetric, or tagged
    as a @custom_metric or @dataclass. Note that CompoundMetric is neither, so it
    cannot be nested.

    Typically you will need to override namespace(); columnar_update(), calling
    it on the submetrics as needed; and the zero() method to return an
    appropriate "empty" instance of your metric. You will need to override from_protobuf()
    and merge() if your subclass __init__() method takes arguments different than
    CompoundMetrtic's. You can use the submetrics_from_protbuf() and merge_submetrics()
    helper methods to implement them. The CompoundMetric class will handle the rest of
    the Metric interface. Don't use / or : in the subclass' namespace.

    See UnicodeRangeMetric for an example.
    """

    submetrics: Dict[str, Metric]

    def __init__(self, submetrics: Dict[str, Metric]):
        """
        Parameters
        ----------
        submetrics : Dict[str, Metric]
            The collection of metrics that comprise the CompoundMetric.
            The key servers as the name of the sub-metric. E.g., the
            metric summary entries will have keys of the form
               "<namespace>/<submetric name>/<component name>"
            Submetric names should only contain alphanumeric characters,
            hyphens, and underscores.
        """

        if ":" in self.namespace or "/" in self.namespace:
            raise ValueError(f"Invalid namespace {self.namespace}")
        for sub_name, submetric in submetrics.items():
            if ":" in sub_name or "/" in sub_name:
                raise ValueError(f"Invalid submetric name {sub_name}")

        self.submetrics = submetrics.copy()

    def merge_submetrics(self: COMPOUND_METRIC, other: COMPOUND_METRIC) -> Dict[str, Metric]:
        if self.namespace != other.namespace:
            raise ValueError(f"Attempt to merge CompoundMetrics {self.namespace} and {other.namespace}")
        if self.submetrics.keys() != other.submetrics.keys():
            raise ValueError("Attempt to merge incompatible CompoundMetrics")
        return {name: (self.submetrics[name] + other.submetrics[name]) for name in self.submetrics.keys()}

    def merge(self: COMPOUND_METRIC, other: COMPOUND_METRIC) -> COMPOUND_METRIC:
        return self.__class__(self.merge_submetrics(other))

    def to_protobuf(self) -> MetricMessage:
        msg = {}
        for sub_name, submetric in self.submetrics.items():
            sub_msg = submetric.to_protobuf()
            for comp_name, comp_msg in sub_msg.metric_components.items():
                msg[sub_name + ":" + submetric.namespace + "/" + comp_name] = comp_msg
        return MetricMessage(metric_components=msg)

    def get_component_paths(self) -> List[str]:
        res = []
        for sub_name, submetric in self.submetrics.items():
            for comp_name in submetric.get_component_paths():
                res.append(sub_name + ":" + submetric.namespace + "/" + comp_name)
        return res

    def to_summary_dict(self, cfg: SummaryConfig) -> Dict[str, Any]:
        summary = {}
        for sub_name, submetric in self.submetrics.items():
            sub_summary = submetric.to_summary_dict(cfg)
            for comp_name, comp_msg in sub_summary.items():
                summary["/".join([self.namespace, sub_name, comp_name])] = comp_msg
        return summary

    def columnar_update(self, view: PreprocessedColumn) -> OperationResult:
        result = OperationResult()
        for submetric in self.submetrics.values():
            result = result + submetric.columnar_update(view)
        return result

    @classmethod
    def submetrics_from_protobuf(cls: Type[COMPOUND_METRIC], msg: MetricMessage) -> Dict[str, Metric]:
        submetrics: Dict[str, Metric] = {}
        submetric_msgs: Dict[str, Dict[str, MetricComponentMessage]] = {}
        for key, comp_msg in msg.metric_components.items():
            submetric_name, comp_name = key.split("/")
            if submetric_msgs.get(submetric_name) is None:
                submetric_msgs[submetric_name] = {}
            submetric_msgs[submetric_name][comp_name] = comp_msg

        for m_name_and_type, metric_components in submetric_msgs.items():
            m_name, m_type = m_name_and_type.split(":")
            if m_type in _METRIC_DESERIALIZER_REGISTRY:
                metric_class = _METRIC_DESERIALIZER_REGISTRY[m_type]
            else:
                raise UnsupportedError(f"Unsupported metric: {m_type}")

            m_msg = MetricMessage(metric_components=metric_components)
            try:
                submetrics[m_name] = metric_class.from_protobuf(m_msg)
            except:  # noqa
                raise DeserializationError(f"Failed to deserialize metric: {m_name}")
        return submetrics

    @classmethod
    def from_protobuf(cls: Type[COMPOUND_METRIC], msg: MetricMessage) -> COMPOUND_METRIC:
        return cls(cls.submetrics_from_protobuf(msg))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ModelType:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _ModelTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ModelType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _ModelType.ValueType  # 0
    CLASSIFICATION: _ModelType.ValueType  # 1
    REGRESSION: _ModelType.ValueType  # 2
    EMBEDDINGS: _ModelType.ValueType  # 3
    NLP: _ModelType.ValueType  # 4
class ModelType(_ModelType, metaclass=_ModelTypeEnumTypeWrapper):
    pass

UNKNOWN: ModelType.ValueType  # 0
CLASSIFICATION: ModelType.ValueType  # 1
REGRESSION: ModelType.ValueType  # 2
EMBEDDINGS: ModelType.ValueType  # 3
NLP: ModelType.ValueType  # 4
global___ModelType = ModelType


class CountersV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COUNT_FIELD_NUMBER: builtins.int
    TRUE_COUNT_FIELD_NUMBER: builtins.int
    NULL_COUNT_FIELD_NUMBER: builtins.int
    count: builtins.int
    @property
    def true_count(self) -> google.protobuf.wrappers_pb2.Int64Value: ...
    @property
    def null_count(self) -> google.protobuf.wrappers_pb2.Int64Value: ...
    def __init__(self,
        *,
        count: builtins.int = ...,
        true_count: typing.Optional[google.protobuf.wrappers_pb2.Int64Value] = ...,
        null_count: typing.Optional[google.protobuf.wrappers_pb2.Int64Value] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["null_count",b"null_count","true_count",b"true_count"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["count",b"count","null_count",b"null_count","true_count",b"true_count"]) -> None: ...
global___CountersV0 = CountersV0

class InferredType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Type:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[InferredType._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: InferredType._Type.ValueType  # 0
        NULL: InferredType._Type.ValueType  # 1
        FRACTIONAL: InferredType._Type.ValueType  # 2
        INTEGRAL: InferredType._Type.ValueType  # 3
        BOOLEAN: InferredType._Type.ValueType  # 4
        STRING: InferredType._Type.ValueType  # 5
    class Type(_Type, metaclass=_TypeEnumTypeWrapper):
        pass

    UNKNOWN: InferredType.Type.ValueType  # 0
    NULL: InferredType.Type.ValueType  # 1
    FRACTIONAL: InferredType.Type.ValueType  # 2
    INTEGRAL: InferredType.Type.ValueType  # 3
    BOOLEAN: InferredType.Type.ValueType  # 4
    STRING: InferredType.Type.ValueType  # 5

    TYPE_FIELD_NUMBER: builtins.int
    RATIO_FIELD_NUMBER: builtins.int
    type: global___InferredType.Type.ValueType
    ratio: builtins.float
    def __init__(self,
        *,
        type: global___InferredType.Type.ValueType = ...,
        ratio: builtins.float = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ratio",b"ratio","type",b"type"]) -> None: ...
global___InferredType = InferredType

class DoublesMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COUNT_FIELD_NUMBER: builtins.int
    MIN_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    count: builtins.int
    min: builtins.float
    max: builtins.float
    sum: builtins.float
    def __init__(self,
        *,
        count: builtins.int = ...,
        min: builtins.float = ...,
        max: builtins.float = ...,
        sum: builtins.float = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count",b"count","max",b"max","min",b"min","sum",b"sum"]) -> None: ...
global___DoublesMessage = DoublesMessage

class LongsMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COUNT_FIELD_NUMBER: builtins.int
    MIN_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    count: builtins.int
    min: builtins.int
    max: builtins.int
    sum: builtins.int
    def __init__(self,
        *,
        count: builtins.int = ...,
        min: builtins.int = ...,
        max: builtins.int = ...,
        sum: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count",b"count","max",b"max","min",b"min","sum",b"sum"]) -> None: ...
global___LongsMessage = LongsMessage

class VarianceMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COUNT_FIELD_NUMBER: builtins.int
    SUM_FIELD_NUMBER: builtins.int
    MEAN_FIELD_NUMBER: builtins.int
    count: builtins.int
    sum: builtins.float
    """sample variance * (n-1)"""

    mean: builtins.float
    def __init__(self,
        *,
        count: builtins.int = ...,
        sum: builtins.float = ...,
        mean: builtins.float = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count",b"count","mean",b"mean","sum",b"sum"]) -> None: ...
global___VarianceMessage = VarianceMessage

class FrequentNumbersSketchMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SKETCH_FIELD_NUMBER: builtins.int
    LG_MAX_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_max_k: builtins.int
    def __init__(self,
        *,
        sketch: builtins.bytes = ...,
        lg_max_k: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_max_k",b"lg_max_k","sketch",b"sketch"]) -> None: ...
global___FrequentNumbersSketchMessage = FrequentNumbersSketchMessage

class FrequentItemsSketchMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SKETCH_FIELD_NUMBER: builtins.int
    LG_MAX_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_max_k: builtins.int
    def __init__(self,
        *,
        sketch: builtins.bytes = ...,
        lg_max_k: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_max_k",b"lg_max_k","sketch",b"sketch"]) -> None: ...
global___FrequentItemsSketchMessageV0 = FrequentItemsSketchMessageV0

class NumbersMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VARIANCE_FIELD_NUMBER: builtins.int
    DOUBLES_FIELD_NUMBER: builtins.int
    LONGS_FIELD_NUMBER: builtins.int
    HISTOGRAM_FIELD_NUMBER: builtins.int
    THETA_FIELD_NUMBER: builtins.int
    COMPACT_THETA_FIELD_NUMBER: builtins.int
    FREQUENT_NUMBERS_FIELD_NUMBER: builtins.int
    @property
    def variance(self) -> global___VarianceMessage: ...
    @property
    def doubles(self) -> global___DoublesMessage: ...
    @property
    def longs(self) -> global___LongsMessage: ...
    histogram: builtins.bytes
    """sketches"""

    theta: builtins.bytes
    compact_theta: builtins.bytes
    @property
    def frequent_numbers(self) -> global___FrequentNumbersSketchMessage: ...
    def __init__(self,
        *,
        variance: typing.Optional[global___VarianceMessage] = ...,
        doubles: typing.Optional[global___DoublesMessage] = ...,
        longs: typing.Optional[global___LongsMessage] = ...,
        histogram: builtins.bytes = ...,
        theta: builtins.bytes = ...,
        compact_theta: builtins.bytes = ...,
        frequent_numbers: typing.Optional[global___FrequentNumbersSketchMessage] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["doubles",b"doubles","frequent_numbers",b"frequent_numbers","longs",b"longs","numbers",b"numbers","variance",b"variance"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["compact_theta",b"compact_theta","doubles",b"doubles","frequent_numbers",b"frequent_numbers","histogram",b"histogram","longs",b"longs","numbers",b"numbers","theta",b"theta","variance",b"variance"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["numbers",b"numbers"]) -> typing.Optional[typing_extensions.Literal["doubles","longs"]]: ...
global___NumbersMessageV0 = NumbersMessageV0

class CharPosMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class CharPosMapEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        @property
        def value(self) -> global___NumbersMessageV0: ...
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: typing.Optional[global___NumbersMessageV0] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value",b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    CHAR_LIST_FIELD_NUMBER: builtins.int
    CHAR_POS_MAP_FIELD_NUMBER: builtins.int
    char_list: typing.Text
    @property
    def char_pos_map(self) -> google.protobuf.internal.containers.MessageMap[typing.Text, global___NumbersMessageV0]: ...
    def __init__(self,
        *,
        char_list: typing.Text = ...,
        char_pos_map: typing.Optional[typing.Mapping[typing.Text, global___NumbersMessageV0]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["char_list",b"char_list","char_pos_map",b"char_pos_map"]) -> None: ...
global___CharPosMessage = CharPosMessage

class StringsMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COUNT_FIELD_NUMBER: builtins.int
    THETA_FIELD_NUMBER: builtins.int
    ITEMS_FIELD_NUMBER: builtins.int
    COMPACT_THETA_FIELD_NUMBER: builtins.int
    LENGTH_FIELD_NUMBER: builtins.int
    TOKEN_LENGTH_FIELD_NUMBER: builtins.int
    CHAR_POS_TRACKER_FIELD_NUMBER: builtins.int
    count: builtins.int
    theta: builtins.bytes
    """sketches"""

    items: builtins.bytes
    compact_theta: builtins.bytes
    @property
    def length(self) -> global___NumbersMessageV0: ...
    @property
    def token_length(self) -> global___NumbersMessageV0: ...
    @property
    def char_pos_tracker(self) -> global___CharPosMessage: ...
    def __init__(self,
        *,
        count: builtins.int = ...,
        theta: builtins.bytes = ...,
        items: builtins.bytes = ...,
        compact_theta: builtins.bytes = ...,
        length: typing.Optional[global___NumbersMessageV0] = ...,
        token_length: typing.Optional[global___NumbersMessageV0] = ...,
        char_pos_tracker: typing.Optional[global___CharPosMessage] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["char_pos_tracker",b"char_pos_tracker","length",b"length","token_length",b"token_length"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["char_pos_tracker",b"char_pos_tracker","compact_theta",b"compact_theta","count",b"count","items",b"items","length",b"length","theta",b"theta","token_length",b"token_length"]) -> None: ...
global___StringsMessageV0 = StringsMessageV0

class SchemaMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class TypeCountsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.int
        def __init__(self,
            *,
            key: builtins.int = ...,
            value: builtins.int = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    TYPECOUNTS_FIELD_NUMBER: builtins.int
    INFERRED_TYPE_FIELD_NUMBER: builtins.int
    @property
    def typeCounts(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.int]: ...
    @property
    def inferred_type(self) -> global___InferredType: ...
    def __init__(self,
        *,
        typeCounts: typing.Optional[typing.Mapping[builtins.int, builtins.int]] = ...,
        inferred_type: typing.Optional[global___InferredType] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["inferred_type",b"inferred_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["inferred_type",b"inferred_type","typeCounts",b"typeCounts"]) -> None: ...
global___SchemaMessageV0 = SchemaMessageV0

class ColumnMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_FIELD_NUMBER: builtins.int
    COUNTERS_FIELD_NUMBER: builtins.int
    SCHEMA_FIELD_NUMBER: builtins.int
    NUMBERS_FIELD_NUMBER: builtins.int
    STRINGS_FIELD_NUMBER: builtins.int
    INFERRED_TYPE_FIELD_NUMBER: builtins.int
    FREQUENT_ITEMS_FIELD_NUMBER: builtins.int
    CARDINALITY_TRACKER_FIELD_NUMBER: builtins.int
    name: typing.Text
    @property
    def counters(self) -> global___CountersV0: ...
    @property
    def schema(self) -> global___SchemaMessageV0: ...
    @property
    def numbers(self) -> global___NumbersMessageV0: ...
    @property
    def strings(self) -> global___StringsMessageV0: ...
    @property
    def inferred_type(self) -> global___InferredType: ...
    @property
    def frequent_items(self) -> global___FrequentItemsSketchMessageV0: ...
    @property
    def cardinality_tracker(self) -> global___HllSketchMessageV0: ...
    def __init__(self,
        *,
        name: typing.Text = ...,
        counters: typing.Optional[global___CountersV0] = ...,
        schema: typing.Optional[global___SchemaMessageV0] = ...,
        numbers: typing.Optional[global___NumbersMessageV0] = ...,
        strings: typing.Optional[global___StringsMessageV0] = ...,
        inferred_type: typing.Optional[global___InferredType] = ...,
        frequent_items: typing.Optional[global___FrequentItemsSketchMessageV0] = ...,
        cardinality_tracker: typing.Optional[global___HllSketchMessageV0] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cardinality_tracker",b"cardinality_tracker","counters",b"counters","frequent_items",b"frequent_items","inferred_type",b"inferred_type","numbers",b"numbers","schema",b"schema","strings",b"strings"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cardinality_tracker",b"cardinality_tracker","counters",b"counters","frequent_items",b"frequent_items","inferred_type",b"inferred_type","name",b"name","numbers",b"numbers","schema",b"schema","strings",b"strings"]) -> None: ...
global___ColumnMessageV0 = ColumnMessageV0

class DatasetPropertiesV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        value: typing.Text
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    class MetadataEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        value: typing.Text
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    SCHEMA_MAJOR_VERSION_FIELD_NUMBER: builtins.int
    SCHEMA_MINOR_VERSION_FIELD_NUMBER: builtins.int
    SESSION_ID_FIELD_NUMBER: builtins.int
    SESSION_TIMESTAMP_FIELD_NUMBER: builtins.int
    DATA_TIMESTAMP_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    schema_major_version: builtins.int
    schema_minor_version: builtins.int
    session_id: typing.Text
    session_timestamp: builtins.int
    data_timestamp: builtins.int
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]: ...
    @property
    def metadata(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]:
        """TODO: store other configuration here"""
        pass
    def __init__(self,
        *,
        schema_major_version: builtins.int = ...,
        schema_minor_version: builtins.int = ...,
        session_id: typing.Text = ...,
        session_timestamp: builtins.int = ...,
        data_timestamp: builtins.int = ...,
        tags: typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
        metadata: typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_timestamp",b"data_timestamp","metadata",b"metadata","schema_major_version",b"schema_major_version","schema_minor_version",b"schema_minor_version","session_id",b"session_id","session_timestamp",b"session_timestamp","tags",b"tags"]) -> None: ...
global___DatasetPropertiesV0 = DatasetPropertiesV0

class ScoreMatrixMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LABELS_FIELD_NUMBER: builtins.int
    PREDICTION_FIELD_FIELD_NUMBER: builtins.int
    TARGET_FIELD_FIELD_NUMBER: builtins.int
    SCORE_FIELD_FIELD_NUMBER: builtins.int
    SCORES_FIELD_NUMBER: builtins.int
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    prediction_field: typing.Text
    target_field: typing.Text
    score_field: typing.Text
    @property
    def scores(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NumbersMessageV0]:
        """a flattened NxN matrix (N = len(labels))"""
        pass
    def __init__(self,
        *,
        labels: typing.Optional[typing.Iterable[typing.Text]] = ...,
        prediction_field: typing.Text = ...,
        target_field: typing.Text = ...,
        score_field: typing.Text = ...,
        scores: typing.Optional[typing.Iterable[global___NumbersMessageV0]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["labels",b"labels","prediction_field",b"prediction_field","score_field",b"score_field","scores",b"scores","target_field",b"target_field"]) -> None: ...
global___ScoreMatrixMessage = ScoreMatrixMessage

class RegressionMetricsMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREDICTION_FIELD_FIELD_NUMBER: builtins.int
    TARGET_FIELD_FIELD_NUMBER: builtins.int
    COUNT_FIELD_NUMBER: builtins.int
    SUM_ABS_DIFF_FIELD_NUMBER: builtins.int
    SUM_DIFF_FIELD_NUMBER: builtins.int
    SUM2_DIFF_FIELD_NUMBER: builtins.int
    prediction_field: typing.Text
    target_field: typing.Text
    count: builtins.int
    sum_abs_diff: builtins.float
    sum_diff: builtins.float
    sum2_diff: builtins.float
    def __init__(self,
        *,
        prediction_field: typing.Text = ...,
        target_field: typing.Text = ...,
        count: builtins.int = ...,
        sum_abs_diff: builtins.float = ...,
        sum_diff: builtins.float = ...,
        sum2_diff: builtins.float = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count",b"count","prediction_field",b"prediction_field","sum2_diff",b"sum2_diff","sum_abs_diff",b"sum_abs_diff","sum_diff",b"sum_diff","target_field",b"target_field"]) -> None: ...
global___RegressionMetricsMessage = RegressionMetricsMessage

class ModelMetricsMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SCOREMATRIX_FIELD_NUMBER: builtins.int
    MODELTYPE_FIELD_NUMBER: builtins.int
    REGRESSIONMETRICS_FIELD_NUMBER: builtins.int
    @property
    def scoreMatrix(self) -> global___ScoreMatrixMessage: ...
    modelType: global___ModelType.ValueType
    @property
    def regressionMetrics(self) -> global___RegressionMetricsMessage: ...
    def __init__(self,
        *,
        scoreMatrix: typing.Optional[global___ScoreMatrixMessage] = ...,
        modelType: global___ModelType.ValueType = ...,
        regressionMetrics: typing.Optional[global___RegressionMetricsMessage] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["regressionMetrics",b"regressionMetrics","scoreMatrix",b"scoreMatrix"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["modelType",b"modelType","regressionMetrics",b"regressionMetrics","scoreMatrix",b"scoreMatrix"]) -> None: ...
global___ModelMetricsMessage = ModelMetricsMessage

class ModelProfileMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OUTPUT_FIELDS_FIELD_NUMBER: builtins.int
    METRICS_FIELD_NUMBER: builtins.int
    @property
    def output_fields(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """Reserving fields for ModelMessage"""
        pass
    @property
    def metrics(self) -> global___ModelMetricsMessage: ...
    def __init__(self,
        *,
        output_fields: typing.Optional[typing.Iterable[typing.Text]] = ...,
        metrics: typing.Optional[global___ModelMetricsMessage] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["metrics",b"metrics"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["metrics",b"metrics","output_fields",b"output_fields"]) -> None: ...
global___ModelProfileMessage = ModelProfileMessage

class DatasetProfileMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class ColumnsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text
        @property
        def value(self) -> global___ColumnMessageV0: ...
        def __init__(self,
            *,
            key: typing.Text = ...,
            value: typing.Optional[global___ColumnMessageV0] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value",b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...

    PROPERTIES_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    MODEPROFILE_FIELD_NUMBER: builtins.int
    @property
    def properties(self) -> global___DatasetPropertiesV0: ...
    @property
    def columns(self) -> google.protobuf.internal.containers.MessageMap[typing.Text, global___ColumnMessageV0]: ...
    @property
    def modeProfile(self) -> global___ModelProfileMessage:
        """reserve other fields for dataset level data"""
        pass
    def __init__(self,
        *,
        properties: typing.Optional[global___DatasetPropertiesV0] = ...,
        columns: typing.Optional[typing.Mapping[typing.Text, global___ColumnMessageV0]] = ...,
        modeProfile: typing.Optional[global___ModelProfileMessage] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["modeProfile",b"modeProfile","properties",b"properties"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns",b"columns","modeProfile",b"modeProfile","properties",b"properties"]) -> None: ...
global___DatasetProfileMessageV0 = DatasetProfileMessageV0

class ColumnsChunkSegment(google.protobuf.message.Message):
    """*
    The follow section is for transmission and reconstruction of the dataset
    in WhyLogs backend
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MARKER_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    marker: typing.Text
    """UUID is required to aggregate to the original message
    This should map back to the original dataset
    """

    @property
    def columns(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ColumnMessageV0]: ...
    def __init__(self,
        *,
        marker: typing.Text = ...,
        columns: typing.Optional[typing.Iterable[global___ColumnMessageV0]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns",b"columns","marker",b"marker"]) -> None: ...
global___ColumnsChunkSegment = ColumnsChunkSegment

class DatasetMetadataSegment(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MARKER_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    marker: typing.Text
    @property
    def properties(self) -> global___DatasetPropertiesV0: ...
    def __init__(self,
        *,
        marker: typing.Text = ...,
        properties: typing.Optional[global___DatasetPropertiesV0] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["properties",b"properties"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["marker",b"marker","properties",b"properties"]) -> None: ...
global___DatasetMetadataSegment = DatasetMetadataSegment

class MessageSegment(google.protobuf.message.Message):
    """A segment of a dataset profile. This can be used to composed the
    original object back
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MARKER_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    marker: typing.Text
    @property
    def metadata(self) -> global___DatasetMetadataSegment: ...
    @property
    def columns(self) -> global___ColumnsChunkSegment: ...
    def __init__(self,
        *,
        marker: typing.Text = ...,
        metadata: typing.Optional[global___DatasetMetadataSegment] = ...,
        columns: typing.Optional[global___ColumnsChunkSegment] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["columns",b"columns","item",b"item","metadata",b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns",b"columns","item",b"item","marker",b"marker","metadata",b"metadata"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["item",b"item"]) -> typing.Optional[typing_extensions.Literal["metadata","columns"]]: ...
global___MessageSegment = MessageSegment

class HllSketchMessageV0(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SKETCH_FIELD_NUMBER: builtins.int
    LG_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_k: builtins.int
    def __init__(self,
        *,
        sketch: builtins.bytes = ...,
        lg_k: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_k",b"lg_k","sketch",b"sketch"]) -> None: ...
global___HllSketchMessageV0 = HllSketchMessageV0

class KllFloatsSketchMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SKETCH_FIELD_NUMBER: builtins.int
    LG_K_FIELD_NUMBER: builtins.int
    sketch: builtins.bytes
    lg_k: builtins.int
    def __init__(self,
        *,
        sketch: builtins.bytes = ...,
        lg_k: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["lg_k",b"lg_k","sketch",b"sketch"]) -> None: ...
global___KllFloatsSketchMessage = KllFloatsSketchMessage



import logging
from dataclasses import dataclass

logger = logging.getLogger(__name__)

try:
    import pandas as _pd
except ImportError:  # noqa
    _pd = None  # type: ignore

try:
    import numpy as _np
except ImportError:  # noqa
    _np = None  # type: ignore
    if _pd is not None:
        logger.error("Pandas is installed but numpy is not. Your environment is probably broken.")


@dataclass(frozen=True)
class NumpyStub:
    dtype: None = None
    number: None = None
    floating: None = None
    ndarray: None = None
    timedelta64: None = None
    datetime64: None = None
    unicode_: None = None
    issubdtype: None = None


@dataclass(frozen=True)
class PandasStub(object):
    Series: None = None
    DataFrame: None = None


if _np is None:
    _np = NumpyStub()

if _pd is None:
    _pd = PandasStub()

np = _np
pd = _pd
